      SUBROUTINE DBSPVD (T, K, NDERIV, X, ILEFT, LDVNIK, VNIKX, WORK)           
C***BEGIN PROLOGUE  DBSPVD                                                      
C***PURPOSE  Calculate the value and all derivatives of order less than         
C            NDERIV of all basis functions which do not vanish at X.            
C***LIBRARY   SLATEC                                                            
C***CATEGORY  E3, K6                                                            
C***TYPE      DOUBLE PRECISION (BSPVD-S, DBSPVD-D)                              
C***KEYWORDS  DIFFERENTIATION OF B-SPLINE, EVALUATION OF B-SPLINE               
C***AUTHOR  Amos, D. E., (SNLA)                                                 
C***DESCRIPTION                                                                 
C                                                                               
C     Written by Carl de Boor and modified by D. E. Amos                        
C                                                                               
C     Abstract    **** a double precision routine ****                          
C                                                                               
C         DBSPVD is the BSPLVD routine of the reference.                        
C                                                                               
C         DBSPVD calculates the value and all derivatives of order              
C         less than NDERIV of all basis functions which do not                  
C         (possibly) vanish at X.  ILEFT is input such that                     
C         T(ILEFT) .LE. X .LT. T(ILEFT+1).  A call to INTRV(T,N+1,X,            
C         ILO,ILEFT,MFLAG) will produce the proper ILEFT.  The output of        
C         DBSPVD is a matrix VNIKX(I,J) of dimension at least (K,NDERIV)        
C         whose columns contain the K nonzero basis functions and               
C         their NDERIV-1 right derivatives at X, I=1,K, J=1,NDERIV.             
C         These basis functions have indices ILEFT-K+I, I=1,K,                  
C         K .LE. ILEFT .LE. N.  The nonzero part of the I-th basis              
C         function lies in (T(I),T(I+K)), I=1,N).                               
C                                                                               
C         If X=T(ILEFT+1) then VNIKX contains left limiting values              
C         (left derivatives) at T(ILEFT+1).  In particular, ILEFT = N           
C         produces left limiting values at the right end point                  
C         X=T(N+1).  To obtain left limiting values at T(I), I=K+1,N+1,         
C         set X= next lower distinct knot, call INTRV to get ILEFT,             
C         set X=T(I), and then call DBSPVD.                                     
C                                                                               
C     Description of Arguments                                                  
C         Input      T,X are double precision                                   
C          T       - knot vector of length N+K, where                           
C                    N = number of B-spline basis functions                     
C                    N = sum of knot multiplicities-K                           
C          K       - order of the B-spline, K .GE. 1                            
C          NDERIV  - number of derivatives = NDERIV-1,                          
C                    1 .LE. NDERIV .LE. K                                       
C          X       - argument of basis functions,                               
C                    T(K) .LE. X .LE. T(N+1)                                    
C          ILEFT   - largest integer such that                                  
C                    T(ILEFT) .LE. X .LT.  T(ILEFT+1)                           
C          LDVNIK  - leading dimension of matrix VNIKX                          
C                                                                               
C         Output     VNIKX,WORK are double precision                            
C          VNIKX   - matrix of dimension at least (K,NDERIV) contain-           
C                    ing the nonzero basis functions at X and their             
C                    derivatives columnwise.                                    
C          WORK    - a work vector of length (K+1)*(K+2)/2                      
C                                                                               
C     Error Conditions                                                          
C         Improper input is a fatal error                                       
C                                                                               
C***REFERENCES  Carl de Boor, Package for calculating with B-splines,           
C                 SIAM Journal on Numerical Analysis 14, 3 (June 1977),         
C                 pp. 441-472.                                                  
C***ROUTINES CALLED  DBSPVN, XERMSG                                             
C***REVISION HISTORY  (YYMMDD)                                                  
C   800901  DATE WRITTEN                                                        
C   890531  Changed all specific intrinsics to generic.  (WRB)                  
C   890831  Modified array declarations.  (WRB)                                 
C   890831  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)                  
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  DBSPVD                                                        
C                                                                               
      INTEGER I,IDERIV,ILEFT,IPKMD,J,JJ,JLOW,JM,JP1MID,K,KMD, KP1, L,           
     1 LDUMMY, M, MHIGH, NDERIV                                                 
      DOUBLE PRECISION FACTOR, FKMD, T, V, VNIKX, WORK, X                       
C     DIMENSION T(ILEFT+K), WORK((K+1)*(K+2)/2)                                 
C     A(I,J) = WORK(I+J*(J+1)/2),  I=1,J+1  J=1,K-1                             
C     A(I,K) = W0RK(I+K*(K-1)/2)  I=1.K                                         
C     WORK(1) AND WORK((K+1)*(K+2)/2) ARE NOT USED.                             
      DIMENSION T(*), VNIKX(LDVNIK,*), WORK(*)                                  
C***FIRST EXECUTABLE STATEMENT  DBSPVD                                          
      IF(K.LT.1) GO TO 200                                                      
      IF(NDERIV.LT.1 .OR. NDERIV.GT.K) GO TO 205                                
      IF(LDVNIK.LT.K) GO TO 210                                                 
      IDERIV = NDERIV                                                           
      KP1 = K + 1                                                               
      JJ = KP1 - IDERIV                                                         
      CALL DBSPVN(T, JJ, K, 1, X, ILEFT, VNIKX, WORK, IWORK)                    
      IF (IDERIV.EQ.1) GO TO 100                                                
      MHIGH = IDERIV                                                            
      DO 20 M=2,MHIGH                                                           
        JP1MID = 1                                                              
        DO 10 J=IDERIV,K                                                        
          VNIKX(J,IDERIV) = VNIKX(JP1MID,1)                                     
          JP1MID = JP1MID + 1                                                   
   10   CONTINUE                                                                
        IDERIV = IDERIV - 1                                                     
        JJ = KP1 - IDERIV                                                       
        CALL DBSPVN(T, JJ, K, 2, X, ILEFT, VNIKX, WORK, IWORK)                  
   20 CONTINUE                                                                  
C                                                                               
      JM = KP1*(KP1+1)/2                                                        
      DO 30 L = 1,JM                                                            
        WORK(L) = 0.0D0                                                         
   30 CONTINUE                                                                  
C     A(I,I) = WORK(I*(I+3)/2) = 1.0       I = 1,K                              
      L = 2                                                                     
      J = 0                                                                     
      DO 40 I = 1,K                                                             
        J = J + L                                                               
        WORK(J) = 1.0D0                                                         
        L = L + 1                                                               
   40 CONTINUE                                                                  
      KMD = K                                                                   
      DO 90 M=2,MHIGH                                                           
        KMD = KMD - 1                                                           
        FKMD = KMD                                                              
        I = ILEFT                                                               
        J = K                                                                   
        JJ = J*(J+1)/2                                                          
        JM = JJ - J                                                             
        DO 60 LDUMMY=1,KMD                                                      
          IPKMD = I + KMD                                                       
          FACTOR = FKMD/(T(IPKMD)-T(I))                                         
          DO 50 L=1,J                                                           
            WORK(L+JJ) = (WORK(L+JJ)-WORK(L+JM))*FACTOR                         
   50     CONTINUE                                                              
          I = I - 1                                                             
          J = J - 1                                                             
          JJ = JM                                                               
          JM = JM - J                                                           
   60   CONTINUE                                                                
C                                                                               
        DO 80 I=1,K                                                             
          V = 0.0D0                                                             
          JLOW = MAX(I,M)                                                       
          JJ = JLOW*(JLOW+1)/2                                                  
          DO 70 J=JLOW,K                                                        
            V = WORK(I+JJ)*VNIKX(J,M) + V                                       
            JJ = JJ + J + 1                                                     
   70     CONTINUE                                                              
          VNIKX(I,M) = V                                                        
   80   CONTINUE                                                                
   90 CONTINUE                                                                  
  100 RETURN                                                                    
C                                                                               
C                                                                               
  200 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVD', 'K DOES NOT SATISFY K.GE.1', 2,          
     +   1)                                                                     
      RETURN                                                                    
  205 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVD',                                          
     +   'NDERIV DOES NOT SATISFY 1.LE.NDERIV.LE.K', 2, 1)                      
      RETURN                                                                    
  210 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVD',                                          
     +   'LDVNIK DOES NOT SATISFY LDVNIK.GE.K', 2, 1)                           
      RETURN                                                                    
      END                                                                       
      SUBROUTINE DBSPVN (T, JHIGH, K, INDEX, X, ILEFT, VNIKX, WORK,             
     +   IWORK)                                                                 
C***BEGIN PROLOGUE  DBSPVN                                                      
C***PURPOSE  Calculate the value of all (possibly) nonzero basis                
C            functions at X.                                                    
C***LIBRARY   SLATEC                                                            
C***CATEGORY  E3, K6                                                            
C***TYPE      DOUBLE PRECISION (BSPVN-S, DBSPVN-D)                              
C***KEYWORDS  EVALUATION OF B-SPLINE                                            
C***AUTHOR  Amos, D. E., (SNLA)                                                 
C***DESCRIPTION                                                                 
C                                                                               
C     Written by Carl de Boor and modified by D. E. Amos                        
C                                                                               
C     Abstract    **** a double precision routine ****                          
C         DBSPVN is the BSPLVN routine of the reference.                        
C                                                                               
C         DBSPVN calculates the value of all (possibly) nonzero basis           
C         functions at X of order MAX(JHIGH,(J+1)*(INDEX-1)), where T(K)        
C         .LE. X .LE. T(N+1) and J=IWORK is set inside the routine on           
C         the first call when INDEX=1.  ILEFT is such that T(ILEFT) .LE.        
C         X .LT. T(ILEFT+1).  A call to DINTRV(T,N+1,X,ILO,ILEFT,MFLAG)         
C         produces the proper ILEFT.  DBSPVN calculates using the basic         
C         algorithm needed in DBSPVD.  If only basis functions are              
C         desired, setting JHIGH=K and INDEX=1 can be faster than               
C         calling DBSPVD, but extra coding is required for derivatives          
C         (INDEX=2) and DBSPVD is set up for this purpose.                      
C                                                                               
C         Left limiting values are set up as described in DBSPVD.               
C                                                                               
C     Description of Arguments                                                  
C                                                                               
C         Input      T,X are double precision                                   
C          T       - knot vector of length N+K, where                           
C                    N = number of B-spline basis functions                     
C                    N = sum of knot multiplicities-K                           
C          JHIGH   - order of B-spline, 1 .LE. JHIGH .LE. K                     
C          K       - highest possible order                                     
C          INDEX   - INDEX = 1 gives basis functions of order JHIGH             
C                          = 2 denotes previous entry with work, IWORK          
C                              values saved for subsequent calls to             
C                              DBSPVN.                                          
C          X       - argument of basis functions,                               
C                    T(K) .LE. X .LE. T(N+1)                                    
C          ILEFT   - largest integer such that                                  
C                    T(ILEFT) .LE. X .LT.  T(ILEFT+1)                           
C                                                                               
C         Output     VNIKX, WORK are double precision                           
C          VNIKX   - vector of length K for spline values.                      
C          WORK    - a work vector of length 2*K                                
C          IWORK   - a work parameter.  Both WORK and IWORK contain             
C                    information necessary to continue for INDEX = 2.           
C                    When INDEX = 1 exclusively, these are scratch              
C                    variables and can be used for other purposes.              
C                                                                               
C     Error Conditions                                                          
C         Improper input is a fatal error.                                      
C                                                                               
C***REFERENCES  Carl de Boor, Package for calculating with B-splines,           
C                 SIAM Journal on Numerical Analysis 14, 3 (June 1977),         
C                 pp. 441-472.                                                  
C***ROUTINES CALLED  XERMSG                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   800901  DATE WRITTEN                                                        
C   890831  Modified array declarations.  (WRB)                                 
C   890831  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)                  
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  DBSPVN                                                        
C                                                                               
      INTEGER ILEFT, IMJP1, INDEX, IPJ, IWORK, JHIGH, JP1, JP1ML, K, L          
      DOUBLE PRECISION T, VM, VMPREV, VNIKX, WORK, X                            
C     DIMENSION T(ILEFT+JHIGH)                                                  
      DIMENSION T(*), VNIKX(*), WORK(*)                                         
C     CONTENT OF J, DELTAM, DELTAP IS EXPECTED UNCHANGED BETWEEN CALLS.         
C     WORK(I) = DELTAP(I), WORK(K+I) = DELTAM(I), I = 1,K                       
C***FIRST EXECUTABLE STATEMENT  DBSPVN                                          
      IF(K.LT.1) GO TO 90                                                       
      IF(JHIGH.GT.K .OR. JHIGH.LT.1) GO TO 100                                  
      IF(INDEX.LT.1 .OR. INDEX.GT.2) GO TO 105                                  
      IF(X.LT.T(ILEFT) .OR. X.GT.T(ILEFT+1)) GO TO 110                          
      GO TO (10, 20), INDEX                                                     
   10 IWORK = 1                                                                 
      VNIKX(1) = 1.0D0                                                          
      IF (IWORK.GE.JHIGH) GO TO 40                                              
C                                                                               
   20 IPJ = ILEFT + IWORK                                                       
      WORK(IWORK) = T(IPJ) - X                                                  
      IMJP1 = ILEFT - IWORK + 1                                                 
      WORK(K+IWORK) = X - T(IMJP1)                                              
      VMPREV = 0.0D0                                                            
      JP1 = IWORK + 1                                                           
      DO 30 L=1,IWORK                                                           
        JP1ML = JP1 - L                                                         
        VM = VNIKX(L)/(WORK(L)+WORK(K+JP1ML))                                   
        VNIKX(L) = VM*WORK(L) + VMPREV                                          
        VMPREV = VM*WORK(K+JP1ML)                                               
   30 CONTINUE                                                                  
      VNIKX(JP1) = VMPREV                                                       
      IWORK = JP1                                                               
      IF (IWORK.LT.JHIGH) GO TO 20                                              
C                                                                               
   40 RETURN                                                                    
C                                                                               
C                                                                               
   90 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVN', 'K DOES NOT SATISFY K.GE.1', 2,          
     +   1)                                                                     
      RETURN                                                                    
  100 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVN',                                          
     +   'JHIGH DOES NOT SATISFY 1.LE.JHIGH.LE.K', 2, 1)                        
      RETURN                                                                    
  105 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVN', 'INDEX IS NOT 1 OR 2', 2, 1)             
      RETURN                                                                    
  110 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBSPVN',                                          
     +   'X DOES NOT SATISFY T(ILEFT).LE.X.LE.T(ILEFT+1)', 2, 1)                
      RETURN                                                                    
      END                                                                       
      DOUBLE PRECISION FUNCTION DGAMLN (Z, IERR)                                
C***BEGIN PROLOGUE  DGAMLN                                                      
C***SUBSIDIARY                                                                  
C***PURPOSE  Compute the logarithm of the Gamma function                        
C***LIBRARY   SLATEC                                                            
C***CATEGORY  C7A                                                               
C***TYPE      DOUBLE PRECISION (GAMLN-S, DGAMLN-D)                              
C***KEYWORDS  LOGARITHM OF GAMMA FUNCTION                                       
C***AUTHOR  Amos, D. E., (SNL)                                                  
C***DESCRIPTION                                                                 
C                                                                               
C               **** A DOUBLE PRECISION ROUTINE ****                            
C         DGAMLN COMPUTES THE NATURAL LOG OF THE GAMMA FUNCTION FOR             
C         Z.GT.0.  THE ASYMPTOTIC EXPANSION IS USED TO GENERATE VALUES          
C         GREATER THAN ZMIN WHICH ARE ADJUSTED BY THE RECURSION                 
C         G(Z+1)=Z*G(Z) FOR Z.LE.ZMIN.  THE FUNCTION WAS MADE AS                
C         PORTABLE AS POSSIBLE BY COMPUTING ZMIN FROM THE NUMBER OF BASE        
C         10 DIGITS IN A WORD, RLN=MAX(-ALOG10(R1MACH(4)),0.5E-18)              
C         LIMITED TO 18 DIGITS OF (RELATIVE) ACCURACY.                          
C                                                                               
C         SINCE INTEGER ARGUMENTS ARE COMMON, A TABLE LOOK UP ON 100            
C         VALUES IS USED FOR SPEED OF EXECUTION.                                
C                                                                               
C     DESCRIPTION OF ARGUMENTS                                                  
C                                                                               
C         INPUT      Z IS D0UBLE PRECISION                                      
C           Z      - ARGUMENT, Z.GT.0.0D0                                       
C                                                                               
C         OUTPUT      DGAMLN IS DOUBLE PRECISION                                
C           DGAMLN  - NATURAL LOG OF THE GAMMA FUNCTION AT Z.NE.0.0D0           
C           IERR    - ERROR FLAG                                                
C                     IERR=0, NORMAL RETURN, COMPUTATION COMPLETED              
C                     IERR=1, Z.LE.0.0D0,    NO COMPUTATION                     
C                                                                               
C                                                                               
C***REFERENCES  COMPUTATION OF BESSEL FUNCTIONS OF COMPLEX ARGUMENT             
C                 BY D. E. AMOS, SAND83-0083, MAY, 1983.                        
C***ROUTINES CALLED  D1MACH, I1MACH                                             
C***REVISION HISTORY  (YYMMDD)                                                  
C   830501  DATE WRITTEN                                                        
C   830501  REVISION DATE from Version 3.2                                      
C   910415  Prologue converted to Version 4.0 format.  (BAB)                    
C   920128  Category corrected.  (WRB)                                          
C   921215  DGAMLN defined for Z negative.  (WRB)                               
C***END PROLOGUE  DGAMLN                                                        
      DOUBLE PRECISION CF, CON, FLN, FZ, GLN, RLN, S, TLG, TRM, TST,            
     * T1, WDTOL, Z, ZDMY, ZINC, ZM, ZMIN, ZP, ZSQ, D1MACH                      
      INTEGER I, IERR, I1M, K, MZ, NZ, I1MACH                                   
      DIMENSION CF(22), GLN(100)                                                
C           LNGAMMA(N), N=1,100                                                 
      DATA GLN(1), GLN(2), GLN(3), GLN(4), GLN(5), GLN(6), GLN(7),              
     1     GLN(8), GLN(9), GLN(10), GLN(11), GLN(12), GLN(13), GLN(14),         
     2     GLN(15), GLN(16), GLN(17), GLN(18), GLN(19), GLN(20),                
     3     GLN(21), GLN(22)/                                                    
     4     0.00000000000000000D+00,     0.00000000000000000D+00,                
     5     6.93147180559945309D-01,     1.79175946922805500D+00,                
     6     3.17805383034794562D+00,     4.78749174278204599D+00,                
     7     6.57925121201010100D+00,     8.52516136106541430D+00,                
     8     1.06046029027452502D+01,     1.28018274800814696D+01,                
     9     1.51044125730755153D+01,     1.75023078458738858D+01,                
     A     1.99872144956618861D+01,     2.25521638531234229D+01,                
     B     2.51912211827386815D+01,     2.78992713838408916D+01,                
     C     3.06718601060806728D+01,     3.35050734501368889D+01,                
     D     3.63954452080330536D+01,     3.93398841871994940D+01,                
     E     4.23356164607534850D+01,     4.53801388984769080D+01/                
      DATA GLN(23), GLN(24), GLN(25), GLN(26), GLN(27), GLN(28),                
     1     GLN(29), GLN(30), GLN(31), GLN(32), GLN(33), GLN(34),                
     2     GLN(35), GLN(36), GLN(37), GLN(38), GLN(39), GLN(40),                
     3     GLN(41), GLN(42), GLN(43), GLN(44)/                                  
     4     4.84711813518352239D+01,     5.16066755677643736D+01,                
     5     5.47847293981123192D+01,     5.80036052229805199D+01,                
     6     6.12617017610020020D+01,     6.45575386270063311D+01,                
     7     6.78897431371815350D+01,     7.12570389671680090D+01,                
     8     7.46582363488301644D+01,     7.80922235533153106D+01,                
     9     8.15579594561150372D+01,     8.50544670175815174D+01,                
     A     8.85808275421976788D+01,     9.21361756036870925D+01,                
     B     9.57196945421432025D+01,     9.93306124547874269D+01,                
     C     1.02968198614513813D+02,     1.06631760260643459D+02,                
     D     1.10320639714757395D+02,     1.14034211781461703D+02,                
     E     1.17771881399745072D+02,     1.21533081515438634D+02/                
      DATA GLN(45), GLN(46), GLN(47), GLN(48), GLN(49), GLN(50),                
     1     GLN(51), GLN(52), GLN(53), GLN(54), GLN(55), GLN(56),                
     2     GLN(57), GLN(58), GLN(59), GLN(60), GLN(61), GLN(62),                
     3     GLN(63), GLN(64), GLN(65), GLN(66)/                                  
     4     1.25317271149356895D+02,     1.29123933639127215D+02,                
     5     1.32952575035616310D+02,     1.36802722637326368D+02,                
     6     1.40673923648234259D+02,     1.44565743946344886D+02,                
     7     1.48477766951773032D+02,     1.52409592584497358D+02,                
     8     1.56360836303078785D+02,     1.60331128216630907D+02,                
     9     1.64320112263195181D+02,     1.68327445448427652D+02,                
     A     1.72352797139162802D+02,     1.76395848406997352D+02,                
     B     1.80456291417543771D+02,     1.84533828861449491D+02,                
     C     1.88628173423671591D+02,     1.92739047287844902D+02,                
     D     1.96866181672889994D+02,     2.01009316399281527D+02,                
     E     2.05168199482641199D+02,     2.09342586752536836D+02/                
      DATA GLN(67), GLN(68), GLN(69), GLN(70), GLN(71), GLN(72),                
     1     GLN(73), GLN(74), GLN(75), GLN(76), GLN(77), GLN(78),                
     2     GLN(79), GLN(80), GLN(81), GLN(82), GLN(83), GLN(84),                
     3     GLN(85), GLN(86), GLN(87), GLN(88)/                                  
     4     2.13532241494563261D+02,     2.17736934113954227D+02,                
     5     2.21956441819130334D+02,     2.26190548323727593D+02,                
     6     2.30439043565776952D+02,     2.34701723442818268D+02,                
     7     2.38978389561834323D+02,     2.43268849002982714D+02,                
     8     2.47572914096186884D+02,     2.51890402209723194D+02,                
     9     2.56221135550009525D+02,     2.60564940971863209D+02,                
     A     2.64921649798552801D+02,     2.69291097651019823D+02,                
     B     2.73673124285693704D+02,     2.78067573440366143D+02,                
     C     2.82474292687630396D+02,     2.86893133295426994D+02,                
     D     2.91323950094270308D+02,     2.95766601350760624D+02,                
     E     3.00220948647014132D+02,     3.04686856765668715D+02/                
      DATA GLN(89), GLN(90), GLN(91), GLN(92), GLN(93), GLN(94),                
     1     GLN(95), GLN(96), GLN(97), GLN(98), GLN(99), GLN(100)/               
     2     3.09164193580146922D+02,     3.13652829949879062D+02,                
     3     3.18152639620209327D+02,     3.22663499126726177D+02,                
     4     3.27185287703775217D+02,     3.31717887196928473D+02,                
     5     3.36261181979198477D+02,     3.40815058870799018D+02,                
     6     3.45379407062266854D+02,     3.49954118040770237D+02,                
     7     3.54539085519440809D+02,     3.59134205369575399D+02/                
C             COEFFICIENTS OF ASYMPTOTIC EXPANSION                              
      DATA CF(1), CF(2), CF(3), CF(4), CF(5), CF(6), CF(7), CF(8),              
     1     CF(9), CF(10), CF(11), CF(12), CF(13), CF(14), CF(15),               
     2     CF(16), CF(17), CF(18), CF(19), CF(20), CF(21), CF(22)/              
     3     8.33333333333333333D-02,    -2.77777777777777778D-03,                
     4     7.93650793650793651D-04,    -5.95238095238095238D-04,                
     5     8.41750841750841751D-04,    -1.91752691752691753D-03,                
     6     6.41025641025641026D-03,    -2.95506535947712418D-02,                
     7     1.79644372368830573D-01,    -1.39243221690590112D+00,                
     8     1.34028640441683920D+01,    -1.56848284626002017D+02,                
     9     2.19310333333333333D+03,    -3.61087712537249894D+04,                
     A     6.91472268851313067D+05,    -1.52382215394074162D+07,                
     B     3.82900751391414141D+08,    -1.08822660357843911D+10,                
     C     3.47320283765002252D+11,    -1.23696021422692745D+13,                
     D     4.88788064793079335D+14,    -2.13203339609193739D+16/                
C                                                                               
C             LN(2*PI)                                                          
      DATA CON                    /     1.83787706640934548D+00/                
C                                                                               
C***FIRST EXECUTABLE STATEMENT  DGAMLN                                          
      IERR=0                                                                    
      IF (Z.LE.0.0D0) GO TO 70                                                  
      IF (Z.GT.101.0D0) GO TO 10                                                
      NZ = Z                                                                    
      FZ = Z - NZ                                                               
      IF (FZ.GT.0.0D0) GO TO 10                                                 
      IF (NZ.GT.100) GO TO 10                                                   
      DGAMLN = GLN(NZ)                                                          
      RETURN                                                                    
   10 CONTINUE                                                                  
      WDTOL = D1MACH(4)                                                         
      WDTOL = MAX(WDTOL,0.5D-18)                                                
      I1M = I1MACH(14)                                                          
      RLN = D1MACH(5)*I1M                                                       
      FLN = MIN(RLN,20.0D0)                                                     
      FLN = MAX(FLN,3.0D0)                                                      
      FLN = FLN - 3.0D0                                                         
      ZM = 1.8000D0 + 0.3875D0*FLN                                              
      MZ = ZM + 1                                                               
      ZMIN = MZ                                                                 
      ZDMY = Z                                                                  
      ZINC = 0.0D0                                                              
      IF (Z.GE.ZMIN) GO TO 20                                                   
      ZINC = ZMIN - NZ                                                          
      ZDMY = Z + ZINC                                                           
   20 CONTINUE                                                                  
      ZP = 1.0D0/ZDMY                                                           
      T1 = CF(1)*ZP                                                             
      S = T1                                                                    
      IF (ZP.LT.WDTOL) GO TO 40                                                 
      ZSQ = ZP*ZP                                                               
      TST = T1*WDTOL                                                            
      DO 30 K=2,22                                                              
        ZP = ZP*ZSQ                                                             
        TRM = CF(K)*ZP                                                          
        IF (ABS(TRM).LT.TST) GO TO 40                                           
        S = S + TRM                                                             
   30 CONTINUE                                                                  
   40 CONTINUE                                                                  
      IF (ZINC.NE.0.0D0) GO TO 50                                               
      TLG = LOG(Z)                                                              
      DGAMLN = Z*(TLG-1.0D0) + 0.5D0*(CON-TLG) + S                              
      RETURN                                                                    
   50 CONTINUE                                                                  
      ZP = 1.0D0                                                                
      NZ = ZINC                                                                 
      DO 60 I=1,NZ                                                              
        ZP = ZP*(Z+(I-1))                                                       
   60 CONTINUE                                                                  
      TLG = LOG(ZDMY)                                                           
      DGAMLN = ZDMY*(TLG-1.0D0) - LOG(ZP) + 0.5D0*(CON-TLG) + S                 
      RETURN                                                                    
C                                                                               
C                                                                               
   70 CONTINUE                                                                  
      DGAMLN = D1MACH(2)                                                        
      IERR=1                                                                    
      RETURN                                                                    
      END                                                                       
      INTEGER FUNCTION I1MACH (I)                                               
C***BEGIN PROLOGUE  I1MACH                                                      
C***PURPOSE  Return integer machine dependent constants.                        
C***LIBRARY   SLATEC                                                            
C***CATEGORY  R1                                                                
C***TYPE      INTEGER (I1MACH-I)                                                
C***KEYWORDS  MACHINE CONSTANTS                                                 
C***AUTHOR  Fox, P. A., (Bell Labs)                                             
C           Hall, A. D., (Bell Labs)                                            
C           Schryer, N. L., (Bell Labs)                                         
C***DESCRIPTION                                                                 
C                                                                               
C   I1MACH can be used to obtain machine-dependent parameters for the           
C   local machine environment.  It is a function subprogram with one            
C   (input) argument and can be referenced as follows:                          
C                                                                               
C        K = I1MACH(I)                                                          
C                                                                               
C   where I=1,...,16.  The (output) value of K above is determined by           
C   the (input) value of I.  The results for various values of I are            
C   discussed below.                                                            
C                                                                               
C   I/O unit numbers:                                                           
C     I1MACH( 1) = the standard input unit.                                     
C     I1MACH( 2) = the standard output unit.                                    
C     I1MACH( 3) = the standard punch unit.                                     
C     I1MACH( 4) = the standard error message unit.                             
C                                                                               
C   Words:                                                                      
C     I1MACH( 5) = the number of bits per integer storage unit.                 
C     I1MACH( 6) = the number of characters per integer storage unit.           
C                                                                               
C   Integers:                                                                   
C     assume integers are represented in the S-digit, base-A form               
C                                                                               
C                sign ( X(S-1)*A**(S-1) + ... + X(1)*A + X(0) )                 
C                                                                               
C                where 0 .LE. X(I) .LT. A for I=0,...,S-1.                      
C     I1MACH( 7) = A, the base.                                                 
C     I1MACH( 8) = S, the number of base-A digits.                              
C     I1MACH( 9) = A**S - 1, the largest magnitude.                             
C                                                                               
C   Floating-Point Numbers:                                                     
C     Assume floating-point numbers are represented in the T-digit,             
C     base-B form                                                               
C                sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )                   
C                                                                               
C                where 0 .LE. X(I) .LT. B for I=1,...,T,                        
C                0 .LT. X(1), and EMIN .LE. E .LE. EMAX.                        
C     I1MACH(10) = B, the base.                                                 
C                                                                               
C   Single-Precision:                                                           
C     I1MACH(11) = T, the number of base-B digits.                              
C     I1MACH(12) = EMIN, the smallest exponent E.                               
C     I1MACH(13) = EMAX, the largest exponent E.                                
C                                                                               
C   Double-Precision:                                                           
C     I1MACH(14) = T, the number of base-B digits.                              
C     I1MACH(15) = EMIN, the smallest exponent E.                               
C     I1MACH(16) = EMAX, the largest exponent E.                                
C                                                                               
C   To alter this function for a particular environment, the desired            
C   set of DATA statements should be activated by removing the C from           
C   column 1.  Also, the values of I1MACH(1) - I1MACH(4) should be              
C   checked for consistency with the local operating system.                    
C                                                                               
C***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for          
C                 a portable library, ACM Transactions on Mathematical          
C                 Software 4, 2 (June 1978), pp. 177-188.                       
C***ROUTINES CALLED  (NONE)                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   750101  DATE WRITTEN                                                        
C   891012  Added VAX G-floating constants.  (WRB)                              
C   891012  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900618  Added DEC RISC constants.  (WRB)                                    
C   900723  Added IBM RS 6000 constants.  (WRB)                                 
C   901009  Correct I1MACH(7) for IBM Mainframes. Should be 2 not 16.           
C           (RWC)                                                               
C   910710  Added HP 730 constants.  (SMR)                                      
C   911114  Added Convex IEEE constants.  (WRB)                                 
C   920121  Added SUN -r8 compiler option constants.  (WRB)                     
C   920229  Added Touchstone Delta i860 constants.  (WRB)                       
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C   920625  Added Convex -p8 and -pd8 compiler option constants.                
C           (BKS, WRB)                                                          
C   930201  Added DEC Alpha and SGI constants.  (RWC and WRB)                   
C   930618  Corrected I1MACH(5) for Convex -p8 and -pd8 compiler                
C           options.  (DWL, RWC and WRB).                                       
C***END PROLOGUE  I1MACH                                                        
C                                                                               
      INTEGER IMACH(16),OUTPUT                                                  
      SAVE IMACH                                                                
      EQUIVALENCE (IMACH(4),OUTPUT)                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE AMIGA                                           
C     ABSOFT COMPILER                                                           
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -126 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1022 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE APOLLO                                          
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        129 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1025 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM                           
C                                                                               
C     DATA IMACH( 1) /          7 /                                             
C     DATA IMACH( 2) /          2 /                                             
C     DATA IMACH( 3) /          2 /                                             
C     DATA IMACH( 4) /          2 /                                             
C     DATA IMACH( 5) /         36 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         33 /                                             
C     DATA IMACH( 9) / Z1FFFFFFFF /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -256 /                                             
C     DATA IMACH(13) /        255 /                                             
C     DATA IMACH(14) /         60 /                                             
C     DATA IMACH(15) /       -256 /                                             
C     DATA IMACH(16) /        255 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE BURROUGHS 5700 SYSTEM                           
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         48 /                                             
C     DATA IMACH( 6) /          6 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         39 /                                             
C     DATA IMACH( 9) / O0007777777777777 /                                      
C     DATA IMACH(10) /          8 /                                             
C     DATA IMACH(11) /         13 /                                             
C     DATA IMACH(12) /        -50 /                                             
C     DATA IMACH(13) /         76 /                                             
C     DATA IMACH(14) /         26 /                                             
C     DATA IMACH(15) /        -50 /                                             
C     DATA IMACH(16) /         76 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE BURROUGHS 6700/7700 SYSTEMS                     
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         48 /                                             
C     DATA IMACH( 6) /          6 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         39 /                                             
C     DATA IMACH( 9) / O0007777777777777 /                                      
C     DATA IMACH(10) /          8 /                                             
C     DATA IMACH(11) /         13 /                                             
C     DATA IMACH(12) /        -50 /                                             
C     DATA IMACH(13) /         76 /                                             
C     DATA IMACH(14) /         26 /                                             
C     DATA IMACH(15) /     -32754 /                                             
C     DATA IMACH(16) /      32780 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CDC 170/180 SERIES USING NOS/VE                 
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         64 /                                             
C     DATA IMACH( 6) /          8 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         63 /                                             
C     DATA IMACH( 9) / 9223372036854775807 /                                    
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         47 /                                             
C     DATA IMACH(12) /      -4095 /                                             
C     DATA IMACH(13) /       4094 /                                             
C     DATA IMACH(14) /         94 /                                             
C     DATA IMACH(15) /      -4095 /                                             
C     DATA IMACH(16) /       4094 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CDC 6000/7000 SERIES                            
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /    6LOUTPUT/                                             
C     DATA IMACH( 5) /         60 /                                             
C     DATA IMACH( 6) /         10 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         48 /                                             
C     DATA IMACH( 9) / 00007777777777777777B /                                  
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         47 /                                             
C     DATA IMACH(12) /       -929 /                                             
C     DATA IMACH(13) /       1070 /                                             
C     DATA IMACH(14) /         94 /                                             
C     DATA IMACH(15) /       -929 /                                             
C     DATA IMACH(16) /       1069 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CELERITY C1260                                  
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          0 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / Z'7FFFFFFF' /                                            
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -126 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1022 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -fn COMPILER OPTION                                             
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1023 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -fi COMPILER OPTION                                             
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -p8 COMPILER OPTION                                             
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         64 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         63 /                                             
C     DATA IMACH( 9) / 9223372036854775807 /                                    
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         53 /                                             
C     DATA IMACH(12) /      -1023 /                                             
C     DATA IMACH(13) /       1023 /                                             
C     DATA IMACH(14) /        113 /                                             
C     DATA IMACH(15) /     -16383 /                                             
C     DATA IMACH(16) /      16383 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -pd8 COMPILER OPTION                                            
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         64 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         63 /                                             
C     DATA IMACH( 9) / 9223372036854775807 /                                    
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         53 /                                             
C     DATA IMACH(12) /      -1023 /                                             
C     DATA IMACH(13) /       1023 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1023 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CRAY                                            
C     USING THE 46 BIT INTEGER COMPILER OPTION                                  
C                                                                               
C     DATA IMACH( 1) /        100 /                                             
C     DATA IMACH( 2) /        101 /                                             
C     DATA IMACH( 3) /        102 /                                             
C     DATA IMACH( 4) /        101 /                                             
C     DATA IMACH( 5) /         64 /                                             
C     DATA IMACH( 6) /          8 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         46 /                                             
C     DATA IMACH( 9) / 1777777777777777B /                                      
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         47 /                                             
C     DATA IMACH(12) /      -8189 /                                             
C     DATA IMACH(13) /       8190 /                                             
C     DATA IMACH(14) /         94 /                                             
C     DATA IMACH(15) /      -8099 /                                             
C     DATA IMACH(16) /       8190 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE CRAY                                            
C     USING THE 64 BIT INTEGER COMPILER OPTION                                  
C                                                                               
C     DATA IMACH( 1) /        100 /                                             
C     DATA IMACH( 2) /        101 /                                             
C     DATA IMACH( 3) /        102 /                                             
C     DATA IMACH( 4) /        101 /                                             
C     DATA IMACH( 5) /         64 /                                             
C     DATA IMACH( 6) /          8 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         63 /                                             
C     DATA IMACH( 9) / 777777777777777777777B /                                 
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         47 /                                             
C     DATA IMACH(12) /      -8189 /                                             
C     DATA IMACH(13) /       8190 /                                             
C     DATA IMACH(14) /         94 /                                             
C     DATA IMACH(15) /      -8099 /                                             
C     DATA IMACH(16) /       8190 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200                      
C                                                                               
C     DATA IMACH( 1) /         11 /                                             
C     DATA IMACH( 2) /         12 /                                             
C     DATA IMACH( 3) /          8 /                                             
C     DATA IMACH( 4) /         10 /                                             
C     DATA IMACH( 5) /         16 /                                             
C     DATA IMACH( 6) /          2 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         15 /                                             
C     DATA IMACH( 9) /      32767 /                                             
C     DATA IMACH(10) /         16 /                                             
C     DATA IMACH(11) /          6 /                                             
C     DATA IMACH(12) /        -64 /                                             
C     DATA IMACH(13) /         63 /                                             
C     DATA IMACH(14) /         14 /                                             
C     DATA IMACH(15) /        -64 /                                             
C     DATA IMACH(16) /         63 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC ALPHA                                       
C     USING G_FLOAT                                                             
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1023 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC ALPHA                                       
C     USING IEEE_FLOAT                                                          
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC RISC                                        
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC VAX                                         
C     USING D_FLOATING                                                          
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         56 /                                             
C     DATA IMACH(15) /       -127 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC VAX                                         
C     USING G_FLOATING                                                          
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1023 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE ELXSI 6400                                      
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         32 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -126 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1022 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE HARRIS 220                                      
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          0 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         24 /                                             
C     DATA IMACH( 6) /          3 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         23 /                                             
C     DATA IMACH( 9) /    8388607 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         23 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         38 /                                             
C     DATA IMACH(15) /       -127 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE HONEYWELL 600/6000 SERIES                       
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /         43 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         36 /                                             
C     DATA IMACH( 6) /          6 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         35 /                                             
C     DATA IMACH( 9) / O377777777777 /                                          
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         27 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         63 /                                             
C     DATA IMACH(15) /       -127 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 730                                          
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 2100                                         
C     3 WORD DOUBLE PRECISION OPTION WITH FTN4                                  
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          4 /                                             
C     DATA IMACH( 4) /          1 /                                             
C     DATA IMACH( 5) /         16 /                                             
C     DATA IMACH( 6) /          2 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         15 /                                             
C     DATA IMACH( 9) /      32767 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         23 /                                             
C     DATA IMACH(12) /       -128 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         39 /                                             
C     DATA IMACH(15) /       -128 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 2100                                         
C     4 WORD DOUBLE PRECISION OPTION WITH FTN4                                  
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          4 /                                             
C     DATA IMACH( 4) /          1 /                                             
C     DATA IMACH( 5) /         16 /                                             
C     DATA IMACH( 6) /          2 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         15 /                                             
C     DATA IMACH( 9) /      32767 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         23 /                                             
C     DATA IMACH(12) /       -128 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         55 /                                             
C     DATA IMACH(15) /       -128 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 9000                                         
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          7 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         32 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -126 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1015 /                                             
C     DATA IMACH(16) /       1017 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,                             
C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86, AND                         
C     THE PERKIN ELMER (INTERDATA) 7/32.                                        
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          7 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) /  Z7FFFFFFF /                                             
C     DATA IMACH(10) /         16 /                                             
C     DATA IMACH(11) /          6 /                                             
C     DATA IMACH(12) /        -64 /                                             
C     DATA IMACH(13) /         63 /                                             
C     DATA IMACH(14) /         14 /                                             
C     DATA IMACH(15) /        -64 /                                             
C     DATA IMACH(16) /         63 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE IBM PC                                          
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          0 /                                             
C     DATA IMACH( 4) /          0 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1023 /                                             
C     DATA IMACH/5,6,0,6,32,4,2,31,2147483647,2,24,                             
C    * -125,127,53,-1021,1023/                                                  
C               NOTE! I1MACH(3) IS NOT WELL DEFINED AND IS SET TO ZERO.         
C                                                                               
C     MACHINE CONSTANTS FOR THE IBM RS 6000                                     
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          0 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE INTEL i860                                      
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE PDP-10 (KA PROCESSOR)                           
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         36 /                                             
C     DATA IMACH( 6) /          5 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         35 /                                             
C     DATA IMACH( 9) / "377777777777 /                                          
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         27 /                                             
C     DATA IMACH(12) /       -128 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         54 /                                             
C     DATA IMACH(15) /       -101 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE PDP-10 (KI PROCESSOR)                           
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         36 /                                             
C     DATA IMACH( 6) /          5 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         35 /                                             
C     DATA IMACH( 9) / "377777777777 /                                          
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         27 /                                             
C     DATA IMACH(12) /       -128 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         62 /                                             
C     DATA IMACH(15) /       -128 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING                           
C     32-BIT INTEGER ARITHMETIC.                                                
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         56 /                                             
C     DATA IMACH(15) /       -127 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING                           
C     16-BIT INTEGER ARITHMETIC.                                                
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          5 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         16 /                                             
C     DATA IMACH( 6) /          2 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         15 /                                             
C     DATA IMACH( 9) /      32767 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         56 /                                             
C     DATA IMACH(15) /       -127 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS                                
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -125 /                                             
C     DATA IMACH(13) /        128 /                                             
C     DATA IMACH(14) /         53 /                                             
C     DATA IMACH(15) /      -1021 /                                             
C     DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE SUN                                             
C                                                                               
      DATA IMACH( 1) /          5 /                                             
      DATA IMACH( 2) /          6 /                                             
      DATA IMACH( 3) /          6 /                                             
      DATA IMACH( 4) /          6 /                                             
      DATA IMACH( 5) /         32 /                                             
      DATA IMACH( 6) /          4 /                                             
      DATA IMACH( 7) /          2 /                                             
      DATA IMACH( 8) /         31 /                                             
      DATA IMACH( 9) / 2147483647 /                                             
      DATA IMACH(10) /          2 /                                             
      DATA IMACH(11) /         24 /                                             
      DATA IMACH(12) /       -125 /                                             
      DATA IMACH(13) /        128 /                                             
      DATA IMACH(14) /         53 /                                             
      DATA IMACH(15) /      -1021 /                                             
      DATA IMACH(16) /       1024 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE SUN                                             
C     USING THE -r8 COMPILER OPTION                                             
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          6 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         32 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         31 /                                             
C     DATA IMACH( 9) / 2147483647 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         53 /                                             
C     DATA IMACH(12) /      -1021 /                                             
C     DATA IMACH(13) /       1024 /                                             
C     DATA IMACH(14) /        113 /                                             
C     DATA IMACH(15) /     -16381 /                                             
C     DATA IMACH(16) /      16384 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES FTN COMPILER                 
C                                                                               
C     DATA IMACH( 1) /          5 /                                             
C     DATA IMACH( 2) /          6 /                                             
C     DATA IMACH( 3) /          1 /                                             
C     DATA IMACH( 4) /          6 /                                             
C     DATA IMACH( 5) /         36 /                                             
C     DATA IMACH( 6) /          4 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         35 /                                             
C     DATA IMACH( 9) / O377777777777 /                                          
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         27 /                                             
C     DATA IMACH(12) /       -128 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         60 /                                             
C     DATA IMACH(15) /      -1024 /                                             
C     DATA IMACH(16) /       1023 /                                             
C                                                                               
C     MACHINE CONSTANTS FOR THE Z80 MICROPROCESSOR                              
C                                                                               
C     DATA IMACH( 1) /          1 /                                             
C     DATA IMACH( 2) /          1 /                                             
C     DATA IMACH( 3) /          0 /                                             
C     DATA IMACH( 4) /          1 /                                             
C     DATA IMACH( 5) /         16 /                                             
C     DATA IMACH( 6) /          2 /                                             
C     DATA IMACH( 7) /          2 /                                             
C     DATA IMACH( 8) /         15 /                                             
C     DATA IMACH( 9) /      32767 /                                             
C     DATA IMACH(10) /          2 /                                             
C     DATA IMACH(11) /         24 /                                             
C     DATA IMACH(12) /       -127 /                                             
C     DATA IMACH(13) /        127 /                                             
C     DATA IMACH(14) /         56 /                                             
C     DATA IMACH(15) /       -127 /                                             
C     DATA IMACH(16) /        127 /                                             
C                                                                               
C***FIRST EXECUTABLE STATEMENT  I1MACH                                          
      IF (I .LT. 1  .OR.  I .GT. 16) GO TO 10                                   
C                                                                               
      I1MACH = IMACH(I)                                                         
      RETURN                                                                    
C                                                                               
   10 CONTINUE                                                                  
      WRITE (UNIT = OUTPUT, FMT = 9000)                                         
 9000 FORMAT ('1ERROR    1 IN I1MACH - I OUT OF BOUNDS')                        
C                                                                               
C     CALL FDUMP                                                                
C                                                                               
      STOP                                                                      
      END                                                                       
      DOUBLE PRECISION FUNCTION D1MACH (I)                                      
C***BEGIN PROLOGUE  D1MACH                                                      
C***PURPOSE  Return floating point machine dependent constants.                 
C***LIBRARY   SLATEC                                                            
C***CATEGORY  R1                                                                
C***TYPE      DOUBLE PRECISION (R1MACH-S, D1MACH-D)                             
C***KEYWORDS  MACHINE CONSTANTS                                                 
C***AUTHOR  Fox, P. A., (Bell Labs)                                             
C           Hall, A. D., (Bell Labs)                                            
C           Schryer, N. L., (Bell Labs)                                         
C***DESCRIPTION                                                                 
C                                                                               
C   D1MACH can be used to obtain machine-dependent parameters for the           
C   local machine environment.  It is a function subprogram with one            
C   (input) argument, and can be referenced as follows:                         
C                                                                               
C        D = D1MACH(I)                                                          
C                                                                               
C   where I=1,...,5.  The (output) value of D above is determined by            
C   the (input) value of I.  The results for various values of I are            
C   discussed below.                                                            
C                                                                               
C   D1MACH( 1) = B**(EMIN-1), the smallest positive magnitude.                  
C   D1MACH( 2) = B**EMAX*(1 - B**(-T)), the largest magnitude.                  
C   D1MACH( 3) = B**(-T), the smallest relative spacing.                        
C   D1MACH( 4) = B**(1-T), the largest relative spacing.                        
C   D1MACH( 5) = LOG10(B)                                                       
C                                                                               
C   Assume double precision numbers are represented in the T-digit,             
C   base-B form                                                                 
C                                                                               
C              sign (B**E)*( (X(1)/B) + ... + (X(T)/B**T) )                     
C                                                                               
C   where 0 .LE. X(I) .LT. B for I=1,...,T, 0 .LT. X(1), and                    
C   EMIN .LE. E .LE. EMAX.                                                      
C                                                                               
C   The values of B, T, EMIN and EMAX are provided in I1MACH as                 
C   follows:                                                                    
C   I1MACH(10) = B, the base.                                                   
C   I1MACH(14) = T, the number of base-B digits.                                
C   I1MACH(15) = EMIN, the smallest exponent E.                                 
C   I1MACH(16) = EMAX, the largest exponent E.                                  
C                                                                               
C   To alter this function for a particular environment, the desired            
C   set of DATA statements should be activated by removing the C from           
C   column 1.  Also, the values of D1MACH(1) - D1MACH(4) should be              
C   checked for consistency with the local operating system.                    
C                                                                               
C***REFERENCES  P. A. Fox, A. D. Hall and N. L. Schryer, Framework for          
C                 a portable library, ACM Transactions on Mathematical          
C                 Software 4, 2 (June 1978), pp. 177-188.                       
C***ROUTINES CALLED  XERMSG                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   750101  DATE WRITTEN                                                        
C   890213  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)                  
C   900618  Added DEC RISC constants.  (WRB)                                    
C   900723  Added IBM RS 6000 constants.  (WRB)                                 
C   900911  Added SUN 386i constants.  (WRB)                                    
C   910710  Added HP 730 constants.  (SMR)                                      
C   911114  Added Convex IEEE constants.  (WRB)                                 
C   920121  Added SUN -r8 compiler option constants.  (WRB)                     
C   920229  Added Touchstone Delta i860 constants.  (WRB)                       
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C   920625  Added CONVEX -p8 and -pd8 compiler option constants.                
C           (BKS, WRB)                                                          
C   930201  Added DEC Alpha and SGI constants.  (RWC and WRB)                   
C***END PROLOGUE  D1MACH                                                        
C                                                                               
      INTEGER SMALL(4)                                                          
      INTEGER LARGE(4)                                                          
      INTEGER RIGHT(4)                                                          
      INTEGER DIVER(4)                                                          
      INTEGER LOG10(4)                                                          
C                                                                               
      DOUBLE PRECISION DMACH(5)                                                 
      SAVE DMACH                                                                
C                                                                               
      EQUIVALENCE (DMACH(1),SMALL(1))                                           
      EQUIVALENCE (DMACH(2),LARGE(1))                                           
      EQUIVALENCE (DMACH(3),RIGHT(1))                                           
      EQUIVALENCE (DMACH(4),DIVER(1))                                           
      EQUIVALENCE (DMACH(5),LOG10(1))                                           
C                                                                               
C     MACHINE CONSTANTS FOR THE AMIGA                                           
C     ABSOFT FORTRAN COMPILER USING THE 68020/68881 COMPILER OPTION             
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /                      
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /                      
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /                      
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /                      
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /                      
C                                                                               
C     MACHINE CONSTANTS FOR THE AMIGA                                           
C     ABSOFT FORTRAN COMPILER USING SOFTWARE FLOATING POINT                     
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /                      
C     DATA LARGE(1), LARGE(2) / Z'7FDFFFFF', Z'FFFFFFFF' /                      
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /                      
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /                      
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /                      
C                                                                               
C     MACHINE CONSTANTS FOR THE APOLLO                                          
C                                                                               
C     DATA SMALL(1), SMALL(2) / 16#00100000, 16#00000000 /                      
C     DATA LARGE(1), LARGE(2) / 16#7FFFFFFF, 16#FFFFFFFF /                      
C     DATA RIGHT(1), RIGHT(2) / 16#3CA00000, 16#00000000 /                      
C     DATA DIVER(1), DIVER(2) / 16#3CB00000, 16#00000000 /                      
C     DATA LOG10(1), LOG10(2) / 16#3FD34413, 16#509F79FF /                      
C                                                                               
C     MACHINE CONSTANTS FOR THE BURROUGHS 1700 SYSTEM                           
C                                                                               
C     DATA SMALL(1) / ZC00800000 /                                              
C     DATA SMALL(2) / Z000000000 /                                              
C     DATA LARGE(1) / ZDFFFFFFFF /                                              
C     DATA LARGE(2) / ZFFFFFFFFF /                                              
C     DATA RIGHT(1) / ZCC5800000 /                                              
C     DATA RIGHT(2) / Z000000000 /                                              
C     DATA DIVER(1) / ZCC6800000 /                                              
C     DATA DIVER(2) / Z000000000 /                                              
C     DATA LOG10(1) / ZD00E730E7 /                                              
C     DATA LOG10(2) / ZC77800DC0 /                                              
C                                                                               
C     MACHINE CONSTANTS FOR THE BURROUGHS 5700 SYSTEM                           
C                                                                               
C     DATA SMALL(1) / O1771000000000000 /                                       
C     DATA SMALL(2) / O0000000000000000 /                                       
C     DATA LARGE(1) / O0777777777777777 /                                       
C     DATA LARGE(2) / O0007777777777777 /                                       
C     DATA RIGHT(1) / O1461000000000000 /                                       
C     DATA RIGHT(2) / O0000000000000000 /                                       
C     DATA DIVER(1) / O1451000000000000 /                                       
C     DATA DIVER(2) / O0000000000000000 /                                       
C     DATA LOG10(1) / O1157163034761674 /                                       
C     DATA LOG10(2) / O0006677466732724 /                                       
C                                                                               
C     MACHINE CONSTANTS FOR THE BURROUGHS 6700/7700 SYSTEMS                     
C                                                                               
C     DATA SMALL(1) / O1771000000000000 /                                       
C     DATA SMALL(2) / O7770000000000000 /                                       
C     DATA LARGE(1) / O0777777777777777 /                                       
C     DATA LARGE(2) / O7777777777777777 /                                       
C     DATA RIGHT(1) / O1461000000000000 /                                       
C     DATA RIGHT(2) / O0000000000000000 /                                       
C     DATA DIVER(1) / O1451000000000000 /                                       
C     DATA DIVER(2) / O0000000000000000 /                                       
C     DATA LOG10(1) / O1157163034761674 /                                       
C     DATA LOG10(2) / O0006677466732724 /                                       
C                                                                               
C     MACHINE CONSTANTS FOR THE CDC 170/180 SERIES USING NOS/VE                 
C                                                                               
C     DATA SMALL(1) / Z"3001800000000000" /                                     
C     DATA SMALL(2) / Z"3001000000000000" /                                     
C     DATA LARGE(1) / Z"4FFEFFFFFFFFFFFE" /                                     
C     DATA LARGE(2) / Z"4FFE000000000000" /                                     
C     DATA RIGHT(1) / Z"3FD2800000000000" /                                     
C     DATA RIGHT(2) / Z"3FD2000000000000" /                                     
C     DATA DIVER(1) / Z"3FD3800000000000" /                                     
C     DATA DIVER(2) / Z"3FD3000000000000" /                                     
C     DATA LOG10(1) / Z"3FFF9A209A84FBCF" /                                     
C     DATA LOG10(2) / Z"3FFFF7988F8959AC" /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE CDC 6000/7000 SERIES                            
C                                                                               
C     DATA SMALL(1) / 00564000000000000000B /                                   
C     DATA SMALL(2) / 00000000000000000000B /                                   
C     DATA LARGE(1) / 37757777777777777777B /                                   
C     DATA LARGE(2) / 37157777777777777777B /                                   
C     DATA RIGHT(1) / 15624000000000000000B /                                   
C     DATA RIGHT(2) / 00000000000000000000B /                                   
C     DATA DIVER(1) / 15634000000000000000B /                                   
C     DATA DIVER(2) / 00000000000000000000B /                                   
C     DATA LOG10(1) / 17164642023241175717B /                                   
C     DATA LOG10(2) / 16367571421742254654B /                                   
C                                                                               
C     MACHINE CONSTANTS FOR THE CELERITY C1260                                  
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /                      
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /                      
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /                      
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /                      
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /                      
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -fn OR -pd8 COMPILER OPTION                                     
C                                                                               
C     DATA DMACH(1) / Z'0010000000000000' /                                     
C     DATA DMACH(2) / Z'7FFFFFFFFFFFFFFF' /                                     
C     DATA DMACH(3) / Z'3CC0000000000000' /                                     
C     DATA DMACH(4) / Z'3CD0000000000000' /                                     
C     DATA DMACH(5) / Z'3FF34413509F79FF' /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -fi COMPILER OPTION                                             
C                                                                               
C     DATA DMACH(1) / Z'0010000000000000' /                                     
C     DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /                                     
C     DATA DMACH(3) / Z'3CA0000000000000' /                                     
C     DATA DMACH(4) / Z'3CB0000000000000' /                                     
C     DATA DMACH(5) / Z'3FD34413509F79FF' /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE CONVEX                                          
C     USING THE -p8 COMPILER OPTION                                             
C                                                                               
C     DATA DMACH(1) / Z'00010000000000000000000000000000' /                     
C     DATA DMACH(2) / Z'7FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF' /                     
C     DATA DMACH(3) / Z'3F900000000000000000000000000000' /                     
C     DATA DMACH(4) / Z'3F910000000000000000000000000000' /                     
C     DATA DMACH(5) / Z'3FFF34413509F79FEF311F12B35816F9' /                     
C                                                                               
C     MACHINE CONSTANTS FOR THE CRAY                                            
C                                                                               
C     DATA SMALL(1) / 201354000000000000000B /                                  
C     DATA SMALL(2) / 000000000000000000000B /                                  
C     DATA LARGE(1) / 577767777777777777777B /                                  
C     DATA LARGE(2) / 000007777777777777774B /                                  
C     DATA RIGHT(1) / 376434000000000000000B /                                  
C     DATA RIGHT(2) / 000000000000000000000B /                                  
C     DATA DIVER(1) / 376444000000000000000B /                                  
C     DATA DIVER(2) / 000000000000000000000B /                                  
C     DATA LOG10(1) / 377774642023241175717B /                                  
C     DATA LOG10(2) / 000007571421742254654B /                                  
C                                                                               
C     MACHINE CONSTANTS FOR THE DATA GENERAL ECLIPSE S/200                      
C     NOTE - IT MAY BE APPROPRIATE TO INCLUDE THE FOLLOWING CARD -              
C     STATIC DMACH(5)                                                           
C                                                                               
C     DATA SMALL /    20K, 3*0 /                                                
C     DATA LARGE / 77777K, 3*177777K /                                          
C     DATA RIGHT / 31420K, 3*0 /                                                
C     DATA DIVER / 32020K, 3*0 /                                                
C     DATA LOG10 / 40423K, 42023K, 50237K, 74776K /                             
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC ALPHA                                       
C     USING G_FLOAT                                                             
C                                                                               
C     DATA DMACH(1) / '0000000000000010'X /                                     
C     DATA DMACH(2) / 'FFFFFFFFFFFF7FFF'X /                                     
C     DATA DMACH(3) / '0000000000003CC0'X /                                     
C     DATA DMACH(4) / '0000000000003CD0'X /                                     
C     DATA DMACH(5) / '79FF509F44133FF3'X /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC ALPHA                                       
C     USING IEEE_FORMAT                                                         
C                                                                               
C     DATA DMACH(1) / '0010000000000000'X /                                     
C     DATA DMACH(2) / '7FEFFFFFFFFFFFFF'X /                                     
C     DATA DMACH(3) / '3CA0000000000000'X /                                     
C     DATA DMACH(4) / '3CB0000000000000'X /                                     
C     DATA DMACH(5) / '3FD34413509F79FF'X /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC RISC                                        
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z'00000000', Z'00100000'/                       
C     DATA LARGE(1), LARGE(2) / Z'FFFFFFFF', Z'7FEFFFFF'/                       
C     DATA RIGHT(1), RIGHT(2) / Z'00000000', Z'3CA00000'/                       
C     DATA DIVER(1), DIVER(2) / Z'00000000', Z'3CB00000'/                       
C     DATA LOG10(1), LOG10(2) / Z'509F79FF', Z'3FD34413'/                       
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC VAX                                         
C     USING D_FLOATING                                                          
C     (EXPRESSED IN INTEGER AND HEXADECIMAL)                                    
C     THE HEX FORMAT BELOW MAY NOT BE SUITABLE FOR UNIX SYSTEMS                 
C     THE INTEGER FORMAT SHOULD BE OK FOR UNIX SYSTEMS                          
C                                                                               
C     DATA SMALL(1), SMALL(2) /        128,           0 /                       
C     DATA LARGE(1), LARGE(2) /     -32769,          -1 /                       
C     DATA RIGHT(1), RIGHT(2) /       9344,           0 /                       
C     DATA DIVER(1), DIVER(2) /       9472,           0 /                       
C     DATA LOG10(1), LOG10(2) /  546979738,  -805796613 /                       
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z00000080, Z00000000 /                          
C     DATA LARGE(1), LARGE(2) / ZFFFF7FFF, ZFFFFFFFF /                          
C     DATA RIGHT(1), RIGHT(2) / Z00002480, Z00000000 /                          
C     DATA DIVER(1), DIVER(2) / Z00002500, Z00000000 /                          
C     DATA LOG10(1), LOG10(2) / Z209A3F9A, ZCFF884FB /                          
C                                                                               
C     MACHINE CONSTANTS FOR THE DEC VAX                                         
C     USING G_FLOATING                                                          
C     (EXPRESSED IN INTEGER AND HEXADECIMAL)                                    
C     THE HEX FORMAT BELOW MAY NOT BE SUITABLE FOR UNIX SYSTEMS                 
C     THE INTEGER FORMAT SHOULD BE OK FOR UNIX SYSTEMS                          
C                                                                               
C     DATA SMALL(1), SMALL(2) /         16,           0 /                       
C     DATA LARGE(1), LARGE(2) /     -32769,          -1 /                       
C     DATA RIGHT(1), RIGHT(2) /      15552,           0 /                       
C     DATA DIVER(1), DIVER(2) /      15568,           0 /                       
C     DATA LOG10(1), LOG10(2) /  1142112243, 2046775455 /                       
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z00000010, Z00000000 /                          
C     DATA LARGE(1), LARGE(2) / ZFFFF7FFF, ZFFFFFFFF /                          
C     DATA RIGHT(1), RIGHT(2) / Z00003CC0, Z00000000 /                          
C     DATA DIVER(1), DIVER(2) / Z00003CD0, Z00000000 /                          
C     DATA LOG10(1), LOG10(2) / Z44133FF3, Z79FF509F /                          
C                                                                               
C     MACHINE CONSTANTS FOR THE ELXSI 6400                                      
C     (ASSUMING REAL*8 IS THE DEFAULT DOUBLE PRECISION)                         
C                                                                               
C     DATA SMALL(1), SMALL(2) / '00100000'X,'00000000'X /                       
C     DATA LARGE(1), LARGE(2) / '7FEFFFFF'X,'FFFFFFFF'X /                       
C     DATA RIGHT(1), RIGHT(2) / '3CB00000'X,'00000000'X /                       
C     DATA DIVER(1), DIVER(2) / '3CC00000'X,'00000000'X /                       
C     DATA LOG10(1), LOG10(2) / '3FD34413'X,'509F79FF'X /                       
C                                                                               
C     MACHINE CONSTANTS FOR THE HARRIS 220                                      
C                                                                               
C     DATA SMALL(1), SMALL(2) / '20000000, '00000201 /                          
C     DATA LARGE(1), LARGE(2) / '37777777, '37777577 /                          
C     DATA RIGHT(1), RIGHT(2) / '20000000, '00000333 /                          
C     DATA DIVER(1), DIVER(2) / '20000000, '00000334 /                          
C     DATA LOG10(1), LOG10(2) / '23210115, '10237777 /                          
C                                                                               
C     MACHINE CONSTANTS FOR THE HONEYWELL 600/6000 SERIES                       
C                                                                               
C     DATA SMALL(1), SMALL(2) / O402400000000, O000000000000 /                  
C     DATA LARGE(1), LARGE(2) / O376777777777, O777777777777 /                  
C     DATA RIGHT(1), RIGHT(2) / O604400000000, O000000000000 /                  
C     DATA DIVER(1), DIVER(2) / O606400000000, O000000000000 /                  
C     DATA LOG10(1), LOG10(2) / O776464202324, O117571775714 /                  
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 730                                          
C                                                                               
C     DATA DMACH(1) / Z'0010000000000000' /                                     
C     DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /                                     
C     DATA DMACH(3) / Z'3CA0000000000000' /                                     
C     DATA DMACH(4) / Z'3CB0000000000000' /                                     
C     DATA DMACH(5) / Z'3FD34413509F79FF' /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 2100                                         
C     THREE WORD DOUBLE PRECISION OPTION WITH FTN4                              
C                                                                               
C     DATA SMALL(1), SMALL(2), SMALL(3) / 40000B,       0,       1 /            
C     DATA LARGE(1), LARGE(2), LARGE(3) / 77777B, 177777B, 177776B /            
C     DATA RIGHT(1), RIGHT(2), RIGHT(3) / 40000B,       0,    265B /            
C     DATA DIVER(1), DIVER(2), DIVER(3) / 40000B,       0,    276B /            
C     DATA LOG10(1), LOG10(2), LOG10(3) / 46420B,  46502B,  77777B /            
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 2100                                         
C     FOUR WORD DOUBLE PRECISION OPTION WITH FTN4                               
C                                                                               
C     DATA SMALL(1), SMALL(2) /  40000B,       0 /                              
C     DATA SMALL(3), SMALL(4) /       0,       1 /                              
C     DATA LARGE(1), LARGE(2) /  77777B, 177777B /                              
C     DATA LARGE(3), LARGE(4) / 177777B, 177776B /                              
C     DATA RIGHT(1), RIGHT(2) /  40000B,       0 /                              
C     DATA RIGHT(3), RIGHT(4) /       0,    225B /                              
C     DATA DIVER(1), DIVER(2) /  40000B,       0 /                              
C     DATA DIVER(3), DIVER(4) /       0,    227B /                              
C     DATA LOG10(1), LOG10(2) /  46420B,  46502B /                              
C     DATA LOG10(3), LOG10(4) /  76747B, 176377B /                              
C                                                                               
C     MACHINE CONSTANTS FOR THE HP 9000                                         
C                                                                               
C     DATA SMALL(1), SMALL(2) / 00040000000B, 00000000000B /                    
C     DATA LARGE(1), LARGE(2) / 17737777777B, 37777777777B /                    
C     DATA RIGHT(1), RIGHT(2) / 07454000000B, 00000000000B /                    
C     DATA DIVER(1), DIVER(2) / 07460000000B, 00000000000B /                    
C     DATA LOG10(1), LOG10(2) / 07764642023B, 12047674777B /                    
C                                                                               
C     MACHINE CONSTANTS FOR THE IBM 360/370 SERIES,                             
C     THE XEROX SIGMA 5/7/9, THE SEL SYSTEMS 85/86, AND                         
C     THE PERKIN ELMER (INTERDATA) 7/32.                                        
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z00100000, Z00000000 /                          
C     DATA LARGE(1), LARGE(2) / Z7FFFFFFF, ZFFFFFFFF /                          
C     DATA RIGHT(1), RIGHT(2) / Z33100000, Z00000000 /                          
C     DATA DIVER(1), DIVER(2) / Z34100000, Z00000000 /                          
C     DATA LOG10(1), LOG10(2) / Z41134413, Z509F79FF /                          
C                                                                               
C     MACHINE CONSTANTS FOR THE IBM PC                                          
C     ASSUMES THAT ALL ARITHMETIC IS DONE IN DOUBLE PRECISION                   
C     ON 8088, I.E., NOT IN 80 BIT FORM FOR THE 8087.                           
C                                                                               
C     DATA SMALL(1) / 2.23D-308  /                                              
C     DATA LARGE(1) / 1.79D+308  /                                              
C     DATA RIGHT(1) / 1.11D-16   /                                              
C     DATA DIVER(1) / 2.22D-16   /                                              
C     DATA LOG10(1) / 0.301029995663981195D0 /                                  
C     DATA DMACH/2.23D-308,1.79D+308,1.11D-16,2.22D-16,                         
C    *  0.301029995663981195D0/                                                 
C                                                                               
C     MACHINE CONSTANTS FOR THE IBM RS 6000                                     
C                                                                               
C     DATA DMACH(1) / Z'0010000000000000' /                                     
C     DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /                                     
C     DATA DMACH(3) / Z'3CA0000000000000' /                                     
C     DATA DMACH(4) / Z'3CB0000000000000' /                                     
C     DATA DMACH(5) / Z'3FD34413509F79FF' /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE INTEL i860                                      
C                                                                               
C     DATA DMACH(1) / Z'0010000000000000' /                                     
C     DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /                                     
C     DATA DMACH(3) / Z'3CA0000000000000' /                                     
C     DATA DMACH(4) / Z'3CB0000000000000' /                                     
C     DATA DMACH(5) / Z'3FD34413509F79FF' /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE PDP-10 (KA PROCESSOR)                           
C                                                                               
C     DATA SMALL(1), SMALL(2) / "033400000000, "000000000000 /                  
C     DATA LARGE(1), LARGE(2) / "377777777777, "344777777777 /                  
C     DATA RIGHT(1), RIGHT(2) / "113400000000, "000000000000 /                  
C     DATA DIVER(1), DIVER(2) / "114400000000, "000000000000 /                  
C     DATA LOG10(1), LOG10(2) / "177464202324, "144117571776 /                  
C                                                                               
C     MACHINE CONSTANTS FOR THE PDP-10 (KI PROCESSOR)                           
C                                                                               
C     DATA SMALL(1), SMALL(2) / "000400000000, "000000000000 /                  
C     DATA LARGE(1), LARGE(2) / "377777777777, "377777777777 /                  
C     DATA RIGHT(1), RIGHT(2) / "103400000000, "000000000000 /                  
C     DATA DIVER(1), DIVER(2) / "104400000000, "000000000000 /                  
C     DATA LOG10(1), LOG10(2) / "177464202324, "476747767461 /                  
C                                                                               
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING                           
C     32-BIT INTEGERS (EXPRESSED IN INTEGER AND OCTAL).                         
C                                                                               
C     DATA SMALL(1), SMALL(2) /    8388608,           0 /                       
C     DATA LARGE(1), LARGE(2) / 2147483647,          -1 /                       
C     DATA RIGHT(1), RIGHT(2) /  612368384,           0 /                       
C     DATA DIVER(1), DIVER(2) /  620756992,           0 /                       
C     DATA LOG10(1), LOG10(2) / 1067065498, -2063872008 /                       
C                                                                               
C     DATA SMALL(1), SMALL(2) / O00040000000, O00000000000 /                    
C     DATA LARGE(1), LARGE(2) / O17777777777, O37777777777 /                    
C     DATA RIGHT(1), RIGHT(2) / O04440000000, O00000000000 /                    
C     DATA DIVER(1), DIVER(2) / O04500000000, O00000000000 /                    
C     DATA LOG10(1), LOG10(2) / O07746420232, O20476747770 /                    
C                                                                               
C     MACHINE CONSTANTS FOR PDP-11 FORTRAN SUPPORTING                           
C     16-BIT INTEGERS (EXPRESSED IN INTEGER AND OCTAL).                         
C                                                                               
C     DATA SMALL(1), SMALL(2) /    128,      0 /                                
C     DATA SMALL(3), SMALL(4) /      0,      0 /                                
C     DATA LARGE(1), LARGE(2) /  32767,     -1 /                                
C     DATA LARGE(3), LARGE(4) /     -1,     -1 /                                
C     DATA RIGHT(1), RIGHT(2) /   9344,      0 /                                
C     DATA RIGHT(3), RIGHT(4) /      0,      0 /                                
C     DATA DIVER(1), DIVER(2) /   9472,      0 /                                
C     DATA DIVER(3), DIVER(4) /      0,      0 /                                
C     DATA LOG10(1), LOG10(2) /  16282,   8346 /                                
C     DATA LOG10(3), LOG10(4) / -31493, -12296 /                                
C                                                                               
C     DATA SMALL(1), SMALL(2) / O000200, O000000 /                              
C     DATA SMALL(3), SMALL(4) / O000000, O000000 /                              
C     DATA LARGE(1), LARGE(2) / O077777, O177777 /                              
C     DATA LARGE(3), LARGE(4) / O177777, O177777 /                              
C     DATA RIGHT(1), RIGHT(2) / O022200, O000000 /                              
C     DATA RIGHT(3), RIGHT(4) / O000000, O000000 /                              
C     DATA DIVER(1), DIVER(2) / O022400, O000000 /                              
C     DATA DIVER(3), DIVER(4) / O000000, O000000 /                              
C     DATA LOG10(1), LOG10(2) / O037632, O020232 /                              
C     DATA LOG10(3), LOG10(4) / O102373, O147770 /                              
C                                                                               
C     MACHINE CONSTANTS FOR THE SILICON GRAPHICS                                
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z'00100000', Z'00000000' /                      
C     DATA LARGE(1), LARGE(2) / Z'7FEFFFFF', Z'FFFFFFFF' /                      
C     DATA RIGHT(1), RIGHT(2) / Z'3CA00000', Z'00000000' /                      
C     DATA DIVER(1), DIVER(2) / Z'3CB00000', Z'00000000' /                      
C     DATA LOG10(1), LOG10(2) / Z'3FD34413', Z'509F79FF' /                      
C                                                                               
C     MACHINE CONSTANTS FOR THE SUN                                             
C                                                                               
      DATA DMACH(1) / Z'0010000000000000' /                                     
      DATA DMACH(2) / Z'7FEFFFFFFFFFFFFF' /                                     
      DATA DMACH(3) / Z'3CA0000000000000' /                                     
      DATA DMACH(4) / Z'3CB0000000000000' /                                     
      DATA DMACH(5) / Z'3FD34413509F79FF' /                                     
C                                                                               
C     MACHINE CONSTANTS FOR THE SUN                                             
C     USING THE -r8 COMPILER OPTION                                             
C                                                                               
C     DATA DMACH(1) / Z'00010000000000000000000000000000' /                     
C     DATA DMACH(2) / Z'7FFEFFFFFFFFFFFFFFFFFFFFFFFFFFFF' /                     
C     DATA DMACH(3) / Z'3F8E0000000000000000000000000000' /                     
C     DATA DMACH(4) / Z'3F8F0000000000000000000000000000' /                     
C     DATA DMACH(5) / Z'3FFD34413509F79FEF311F12B35816F9' /                     
C                                                                               
C     MACHINE CONSTANTS FOR THE SUN 386i                                        
C                                                                               
C     DATA SMALL(1), SMALL(2) / Z'FFFFFFFD', Z'000FFFFF' /                      
C     DATA LARGE(1), LARGE(2) / Z'FFFFFFB0', Z'7FEFFFFF' /                      
C     DATA RIGHT(1), RIGHT(2) / Z'000000B0', Z'3CA00000' /                      
C     DATA DIVER(1), DIVER(2) / Z'FFFFFFCB', Z'3CAFFFFF'                        
C     DATA LOG10(1), LOG10(2) / Z'509F79E9', Z'3FD34413' /                      
C                                                                               
C     MACHINE CONSTANTS FOR THE UNIVAC 1100 SERIES FTN COMPILER                 
C                                                                               
C     DATA SMALL(1), SMALL(2) / O000040000000, O000000000000 /                  
C     DATA LARGE(1), LARGE(2) / O377777777777, O777777777777 /                  
C     DATA RIGHT(1), RIGHT(2) / O170540000000, O000000000000 /                  
C     DATA DIVER(1), DIVER(2) / O170640000000, O000000000000 /                  
C     DATA LOG10(1), LOG10(2) / O177746420232, O411757177572 /                  
C                                                                               
C***FIRST EXECUTABLE STATEMENT  D1MACH                                          
      IF (I .LT. 1 .OR. I .GT. 5) CALL XERMSG ('SLATEC', 'D1MACH',              
     +   'I OUT OF BOUNDS', 1, 2)                                               
C                                                                               
      D1MACH = DMACH(I)                                                         
      RETURN                                                                    
C                                                                               
      END                                                                       
      SUBROUTINE XERMSG (LIBRAR, SUBROU, MESSG, NERR, LEVEL)                    
C***BEGIN PROLOGUE  XERMSG                                                      
C***PURPOSE  Process error messages for SLATEC and other libraries.             
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3C                                                               
C***TYPE      ALL (XERMSG-A)                                                    
C***KEYWORDS  ERROR MESSAGE, XERROR                                             
C***AUTHOR  Fong, Kirby, (NMFECC at LLNL)                                       
C***DESCRIPTION                                                                 
C                                                                               
C   XERMSG processes a diagnostic message in a manner determined by the         
C   value of LEVEL and the current value of the library error control           
C   flag, KONTRL.  See subroutine XSETF for details.                            
C                                                                               
C    LIBRAR   A character constant (or character variable) with the name        
C             of the library.  This will be 'SLATEC' for the SLATEC             
C             Common Math Library.  The error handling package is               
C             general enough to be used by many libraries                       
C             simultaneously, so it is desirable for the routine that           
C             detects and reports an error to identify the library name         
C             as well as the routine name.                                      
C                                                                               
C    SUBROU   A character constant (or character variable) with the name        
C             of the routine that detected the error.  Usually it is the        
C             name of the routine that is calling XERMSG.  There are            
C             some instances where a user callable library routine calls        
C             lower level subsidiary routines where the error is                
C             detected.  In such cases it may be more informative to            
C             supply the name of the routine the user called rather than        
C             the name of the subsidiary routine that detected the              
C             error.                                                            
C                                                                               
C    MESSG    A character constant (or character variable) with the text        
C             of the error or warning message.  In the example below,           
C             the message is a character constant that contains a               
C             generic message.                                                  
C                                                                               
C                   CALL XERMSG ('SLATEC', 'MMPY',                              
C                  *'THE ORDER OF THE MATRIX EXCEEDS THE ROW DIMENSION',        
C                  *3, 1)                                                       
C                                                                               
C             It is possible (and is sometimes desirable) to generate a         
C             specific message--e.g., one that contains actual numeric          
C             values.  Specific numeric values can be converted into            
C             character strings using formatted WRITE statements into           
C             character variables.  This is called standard Fortran             
C             internal file I/O and is exemplified in the first three           
C             lines of the following example.  You can also catenate            
C             substrings of characters to construct the error message.          
C             Here is an example showing the use of both writing to             
C             an internal file and catenating character strings.                
C                                                                               
C                   CHARACTER*5 CHARN, CHARL                                    
C                   WRITE (CHARN,10) N                                          
C                   WRITE (CHARL,10) LDA                                        
C                10 FORMAT(I5)                                                  
C                   CALL XERMSG ('SLATEC', 'MMPY', 'THE ORDER'//CHARN//         
C                  *   ' OF THE MATRIX EXCEEDS ITS ROW DIMENSION OF'//          
C                  *   CHARL, 3, 1)                                             
C                                                                               
C             There are two subtleties worth mentioning.  One is that           
C             the // for character catenation is used to construct the          
C             error message so that no single character constant is             
C             continued to the next line.  This avoids confusion as to          
C             whether there are trailing blanks at the end of the line.         
C             The second is that by catenating the parts of the message         
C             as an actual argument rather than encoding the entire             
C             message into one large character variable, we avoid               
C             having to know how long the message will be in order to           
C             declare an adequate length for that large character               
C             variable.  XERMSG calls XERPRN to print the message using         
C             multiple lines if necessary.  If the message is very long,        
C             XERPRN will break it into pieces of 72 characters (as             
C             requested by XERMSG) for printing on multiple lines.              
C             Also, XERMSG asks XERPRN to prefix each line with ' *  '          
C             so that the total line length could be 76 characters.             
C             Note also that XERPRN scans the error message backwards           
C             to ignore trailing blanks.  Another feature is that               
C             the substring '$$' is treated as a new line sentinel              
C             by XERPRN.  If you want to construct a multiline                  
C             message without having to count out multiples of 72               
C             characters, just use '$$' as a separator.  '$$'                   
C             obviously must occur within 72 characters of the                  
C             start of each line to have its intended effect since              
C             XERPRN is asked to wrap around at 72 characters in                
C             addition to looking for '$$'.                                     
C                                                                               
C    NERR     An integer value that is chosen by the library routine's          
C             author.  It must be in the range -99 to 999 (three                
C             printable digits).  Each distinct error should have its           
C             own error number.  These error numbers should be described        
C             in the machine readable documentation for the routine.            
C             The error numbers need be unique only within each routine,        
C             so it is reasonable for each routine to start enumerating         
C             errors from 1 and proceeding to the next integer.                 
C                                                                               
C    LEVEL    An integer value in the range 0 to 2 that indicates the           
C             level (severity) of the error.  Their meanings are                
C                                                                               
C            -1  A warning message.  This is used if it is not clear            
C                that there really is an error, but the user's attention        
C                may be needed.  An attempt is made to only print this          
C                message once.                                                  
C                                                                               
C             0  A warning message.  This is used if it is not clear            
C                that there really is an error, but the user's attention        
C                may be needed.                                                 
C                                                                               
C             1  A recoverable error.  This is used even if the error is        
C                so serious that the routine cannot return any useful           
C                answer.  If the user has told the error package to             
C                return after recoverable errors, then XERMSG will              
C                return to the Library routine which can then return to         
C                the user's routine.  The user may also permit the error        
C                package to terminate the program upon encountering a           
C                recoverable error.                                             
C                                                                               
C             2  A fatal error.  XERMSG will not return to its caller           
C                after it receives a fatal error.  This level should            
C                hardly ever be used; it is much better to allow the            
C                user a chance to recover.  An example of one of the few        
C                cases in which it is permissible to declare a level 2          
C                error is a reverse communication Library routine that          
C                is likely to be called repeatedly until it integrates          
C                across some interval.  If there is a serious error in          
C                the input such that another step cannot be taken and           
C                the Library routine is called again without the input          
C                error having been corrected by the caller, the Library         
C                routine will probably be called forever with improper          
C                input.  In this case, it is reasonable to declare the          
C                error to be fatal.                                             
C                                                                               
C    Each of the arguments to XERMSG is input; none will be modified by         
C    XERMSG.  A routine may make multiple calls to XERMSG with warning          
C    level messages; however, after a call to XERMSG with a recoverable         
C    error, the routine should return to the user.  Do not try to call          
C    XERMSG with a second recoverable error after the first recoverable         
C    error because the error package saves the error number.  The user          
C    can retrieve this error number by calling another entry point in           
C    the error handling package and then clear the error number when            
C    recovering from the error.  Calling XERMSG in succession causes the        
C    old error number to be overwritten by the latest error number.             
C    This is considered harmless for error numbers associated with              
C    warning messages but must not be done for error numbers of serious         
C    errors.  After a call to XERMSG with a recoverable error, the user         
C    must be given a chance to call NUMXER or XERCLR to retrieve or             
C    clear the error number.                                                    
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  FDUMP, J4SAVE, XERCNT, XERHLT, XERPRN, XERSVE              
C***REVISION HISTORY  (YYMMDD)                                                  
C   880101  DATE WRITTEN                                                        
C   880621  REVISED AS DIRECTED AT SLATEC CML MEETING OF FEBRUARY 1988.         
C           THERE ARE TWO BASIC CHANGES.                                        
C           1.  A NEW ROUTINE, XERPRN, IS USED INSTEAD OF XERPRT TO             
C               PRINT MESSAGES.  THIS ROUTINE WILL BREAK LONG MESSAGES          
C               INTO PIECES FOR PRINTING ON MULTIPLE LINES.  '$$' IS            
C               ACCEPTED AS A NEW LINE SENTINEL.  A PREFIX CAN BE               
C               ADDED TO EACH LINE TO BE PRINTED.  XERMSG USES EITHER           
C               ' ***' OR ' *  ' AND LONG MESSAGES ARE BROKEN EVERY             
C               72 CHARACTERS (AT MOST) SO THAT THE MAXIMUM LINE                
C               LENGTH OUTPUT CAN NOW BE AS GREAT AS 76.                        
C           2.  THE TEXT OF ALL MESSAGES IS NOW IN UPPER CASE SINCE THE         
C               FORTRAN STANDARD DOCUMENT DOES NOT ADMIT THE EXISTENCE          
C               OF LOWER CASE.                                                  
C   880708  REVISED AFTER THE SLATEC CML MEETING OF JUNE 29 AND 30.             
C           THE PRINCIPAL CHANGES ARE                                           
C           1.  CLARIFY COMMENTS IN THE PROLOGUES                               
C           2.  RENAME XRPRNT TO XERPRN                                         
C           3.  REWORK HANDLING OF '$$' IN XERPRN TO HANDLE BLANK LINES         
C               SIMILAR TO THE WAY FORMAT STATEMENTS HANDLE THE /               
C               CHARACTER FOR NEW RECORDS.                                      
C   890706  REVISED WITH THE HELP OF FRED FRITSCH AND REG CLEMENS TO            
C           CLEAN UP THE CODING.                                                
C   890721  REVISED TO USE NEW FEATURE IN XERPRN TO COUNT CHARACTERS IN         
C           PREFIX.                                                             
C   891013  REVISED TO CORRECT COMMENTS.                                        
C   891214  Prologue converted to Version 4.0 format.  (WRB)                    
C   900510  Changed test on NERR to be -9999999 < NERR < 99999999, but          
C           NERR .ne. 0, and on LEVEL to be -2 < LEVEL < 3.  Added              
C           LEVEL=-1 logic, changed calls to XERSAV to XERSVE, and              
C           XERCTL to XERCNT.  (RWC)                                            
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  XERMSG                                                        
      CHARACTER*(*) LIBRAR, SUBROU, MESSG                                       
      CHARACTER*8 XLIBR, XSUBR                                                  
      CHARACTER*72  TEMP                                                        
      CHARACTER*20  LFIRST                                                      
C***FIRST EXECUTABLE STATEMENT  XERMSG                                          
      LKNTRL = J4SAVE (2, 0, .FALSE.)                                           
      MAXMES = J4SAVE (4, 0, .FALSE.)                                           
C                                                                               
C       LKNTRL IS A LOCAL COPY OF THE CONTROL FLAG KONTRL.                      
C       MAXMES IS THE MAXIMUM NUMBER OF TIMES ANY PARTICULAR MESSAGE            
C          SHOULD BE PRINTED.                                                   
C                                                                               
C       WE PRINT A FATAL ERROR MESSAGE AND TERMINATE FOR AN ERROR IN            
C          CALLING XERMSG.  THE ERROR NUMBER SHOULD BE POSITIVE,                
C          AND THE LEVEL SHOULD BE BETWEEN 0 AND 2.                             
C                                                                               
      IF (NERR.LT.-9999999 .OR. NERR.GT.99999999 .OR. NERR.EQ.0 .OR.            
     *   LEVEL.LT.-1 .OR. LEVEL.GT.2) THEN                                      
         CALL XERPRN (' ***', -1, 'FATAL ERROR IN...$$ ' //                     
     *      'XERMSG -- INVALID ERROR NUMBER OR LEVEL$$ '//                      
     *      'JOB ABORT DUE TO FATAL ERROR.', 72)                                
         CALL XERSVE (' ', ' ', ' ', 0, 0, 0, KDUMMY)                           
         CALL XERHLT (' ***XERMSG -- INVALID INPUT')                            
         RETURN                                                                 
      ENDIF                                                                     
C                                                                               
C       RECORD THE MESSAGE.                                                     
C                                                                               
      I = J4SAVE (1, NERR, .TRUE.)                                              
      CALL XERSVE (LIBRAR, SUBROU, MESSG, 1, NERR, LEVEL, KOUNT)                
C                                                                               
C       HANDLE PRINT-ONCE WARNING MESSAGES.                                     
C                                                                               
      IF (LEVEL.EQ.-1 .AND. KOUNT.GT.1) RETURN                                  
C                                                                               
C       ALLOW TEMPORARY USER OVERRIDE OF THE CONTROL FLAG.                      
C                                                                               
      XLIBR  = LIBRAR                                                           
      XSUBR  = SUBROU                                                           
      LFIRST = MESSG                                                            
      LERR   = NERR                                                             
      LLEVEL = LEVEL                                                            
      CALL XERCNT (XLIBR, XSUBR, LFIRST, LERR, LLEVEL, LKNTRL)                  
C                                                                               
      LKNTRL = MAX(-2, MIN(2,LKNTRL))                                           
      MKNTRL = ABS(LKNTRL)                                                      
C                                                                               
C       SKIP PRINTING IF THE CONTROL FLAG VALUE AS RESET IN XERCNT IS           
C       ZERO AND THE ERROR IS NOT FATAL.                                        
C                                                                               
      IF (LEVEL.LT.2 .AND. LKNTRL.EQ.0) GO TO 30                                
      IF (LEVEL.EQ.0 .AND. KOUNT.GT.MAXMES) GO TO 30                            
      IF (LEVEL.EQ.1 .AND. KOUNT.GT.MAXMES .AND. MKNTRL.EQ.1) GO TO 30          
      IF (LEVEL.EQ.2 .AND. KOUNT.GT.MAX(1,MAXMES)) GO TO 30                     
C                                                                               
C       ANNOUNCE THE NAMES OF THE LIBRARY AND SUBROUTINE BY BUILDING A          
C       MESSAGE IN CHARACTER VARIABLE TEMP (NOT EXCEEDING 66 CHARACTERS)        
C       AND SENDING IT OUT VIA XERPRN.  PRINT ONLY IF CONTROL FLAG              
C       IS NOT ZERO.                                                            
C                                                                               
      IF (LKNTRL .NE. 0) THEN                                                   
         TEMP(1:21) = 'MESSAGE FROM ROUTINE '                                   
         I = MIN(LEN(SUBROU), 16)                                               
         TEMP(22:21+I) = SUBROU(1:I)                                            
         TEMP(22+I:33+I) = ' IN LIBRARY '                                       
         LTEMP = 33 + I                                                         
         I = MIN(LEN(LIBRAR), 16)                                               
         TEMP(LTEMP+1:LTEMP+I) = LIBRAR (1:I)                                   
         TEMP(LTEMP+I+1:LTEMP+I+1) = '.'                                        
         LTEMP = LTEMP + I + 1                                                  
         CALL XERPRN (' ***', -1, TEMP(1:LTEMP), 72)                            
      ENDIF                                                                     
C                                                                               
C       IF LKNTRL IS POSITIVE, PRINT AN INTRODUCTORY LINE BEFORE                
C       PRINTING THE MESSAGE.  THE INTRODUCTORY LINE TELLS THE CHOICE           
C       FROM EACH OF THE FOLLOWING THREE OPTIONS.                               
C       1.  LEVEL OF THE MESSAGE                                                
C              'INFORMATIVE MESSAGE'                                            
C              'POTENTIALLY RECOVERABLE ERROR'                                  
C              'FATAL ERROR'                                                    
C       2.  WHETHER CONTROL FLAG WILL ALLOW PROGRAM TO CONTINUE                 
C              'PROG CONTINUES'                                                 
C              'PROG ABORTED'                                                   
C       3.  WHETHER OR NOT A TRACEBACK WAS REQUESTED.  (THE TRACEBACK           
C           MAY NOT BE IMPLEMENTED AT SOME SITES, SO THIS ONLY TELLS            
C           WHAT WAS REQUESTED, NOT WHAT WAS DELIVERED.)                        
C              'TRACEBACK REQUESTED'                                            
C              'TRACEBACK NOT REQUESTED'                                        
C       NOTICE THAT THE LINE INCLUDING FOUR PREFIX CHARACTERS WILL NOT          
C       EXCEED 74 CHARACTERS.                                                   
C       WE SKIP THE NEXT BLOCK IF THE INTRODUCTORY LINE IS NOT NEEDED.          
C                                                                               
      IF (LKNTRL .GT. 0) THEN                                                   
C                                                                               
C       THE FIRST PART OF THE MESSAGE TELLS ABOUT THE LEVEL.                    
C                                                                               
         IF (LEVEL .LE. 0) THEN                                                 
            TEMP(1:20) = 'INFORMATIVE MESSAGE,'                                 
            LTEMP = 20                                                          
         ELSEIF (LEVEL .EQ. 1) THEN                                             
            TEMP(1:30) = 'POTENTIALLY RECOVERABLE ERROR,'                       
            LTEMP = 30                                                          
         ELSE                                                                   
            TEMP(1:12) = 'FATAL ERROR,'                                         
            LTEMP = 12                                                          
         ENDIF                                                                  
C                                                                               
C       THEN WHETHER THE PROGRAM WILL CONTINUE.                                 
C                                                                               
         IF ((MKNTRL.EQ.2 .AND. LEVEL.GE.1) .OR.                                
     *       (MKNTRL.EQ.1 .AND. LEVEL.EQ.2)) THEN                               
            TEMP(LTEMP+1:LTEMP+14) = ' PROG ABORTED,'                           
            LTEMP = LTEMP + 14                                                  
         ELSE                                                                   
            TEMP(LTEMP+1:LTEMP+16) = ' PROG CONTINUES,'                         
            LTEMP = LTEMP + 16                                                  
         ENDIF                                                                  
C                                                                               
C       FINALLY TELL WHETHER THERE SHOULD BE A TRACEBACK.                       
C                                                                               
         IF (LKNTRL .GT. 0) THEN                                                
            TEMP(LTEMP+1:LTEMP+20) = ' TRACEBACK REQUESTED'                     
            LTEMP = LTEMP + 20                                                  
         ELSE                                                                   
            TEMP(LTEMP+1:LTEMP+24) = ' TRACEBACK NOT REQUESTED'                 
            LTEMP = LTEMP + 24                                                  
         ENDIF                                                                  
         CALL XERPRN (' ***', -1, TEMP(1:LTEMP), 72)                            
      ENDIF                                                                     
C                                                                               
C       NOW SEND OUT THE MESSAGE.                                               
C                                                                               
      CALL XERPRN (' *  ', -1, MESSG, 72)                                       
C                                                                               
C       IF LKNTRL IS POSITIVE, WRITE THE ERROR NUMBER AND REQUEST A             
C          TRACEBACK.                                                           
C                                                                               
      IF (LKNTRL .GT. 0) THEN                                                   
         WRITE (TEMP, '(''ERROR NUMBER = '', I8)') NERR                         
         DO 10 I=16,22                                                          
            IF (TEMP(I:I) .NE. ' ') GO TO 20                                    
   10    CONTINUE                                                               
C                                                                               
   20    CALL XERPRN (' *  ', -1, TEMP(1:15) // TEMP(I:23), 72)                 
         CALL FDUMP                                                             
      ENDIF                                                                     
C                                                                               
C       IF LKNTRL IS NOT ZERO, PRINT A BLANK LINE AND AN END OF MESSAGE.        
C                                                                               
      IF (LKNTRL .NE. 0) THEN                                                   
         CALL XERPRN (' *  ', -1, ' ', 72)                                      
         CALL XERPRN (' ***', -1, 'END OF MESSAGE', 72)                         
         CALL XERPRN ('    ',  0, ' ', 72)                                      
      ENDIF                                                                     
C                                                                               
C       IF THE ERROR IS NOT FATAL OR THE ERROR IS RECOVERABLE AND THE           
C       CONTROL FLAG IS SET FOR RECOVERY, THEN RETURN.                          
C                                                                               
   30 IF (LEVEL.LE.0 .OR. (LEVEL.EQ.1 .AND. MKNTRL.LE.1)) RETURN                
C                                                                               
C       THE PROGRAM WILL BE STOPPED DUE TO AN UNRECOVERED ERROR OR A            
C       FATAL ERROR.  PRINT THE REASON FOR THE ABORT AND THE ERROR              
C       SUMMARY IF THE CONTROL FLAG AND THE MAXIMUM ERROR COUNT PERMIT.         
C                                                                               
      IF (LKNTRL.GT.0 .AND. KOUNT.LT.MAX(1,MAXMES)) THEN                        
         IF (LEVEL .EQ. 1) THEN                                                 
            CALL XERPRN                                                         
     *         (' ***', -1, 'JOB ABORT DUE TO UNRECOVERED ERROR.', 72)          
         ELSE                                                                   
            CALL XERPRN(' ***', -1, 'JOB ABORT DUE TO FATAL ERROR.', 72)        
         ENDIF                                                                  
         CALL XERSVE (' ', ' ', ' ', -1, 0, 0, KDUMMY)                          
         CALL XERHLT (' ')                                                      
      ELSE                                                                      
         CALL XERHLT (MESSG)                                                    
      ENDIF                                                                     
      RETURN                                                                    
      END                                                                       
      SUBROUTINE FDUMP                                                          
C***BEGIN PROLOGUE  FDUMP                                                       
C***PURPOSE  Symbolic dump (should be locally written).                         
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3                                                                
C***TYPE      ALL (FDUMP-A)                                                     
C***KEYWORDS  ERROR, XERMSG                                                     
C***AUTHOR  Jones, R. E., (SNLA)                                                
C***DESCRIPTION                                                                 
C                                                                               
C        ***Note*** Machine Dependent Routine                                   
C        FDUMP is intended to be replaced by a locally written                  
C        version which produces a symbolic dump.  Failing this,                 
C        it should be replaced by a version which prints the                    
C        subprogram nesting list.  Note that this dump must be                  
C        printed on each of up to five files, as indicated by the               
C        XGETUA routine.  See XSETUA and XGETUA for details.                    
C                                                                               
C     Written by Ron Jones, with SLATEC Common Math Library Subcommittee        
C                                                                               
C***REFERENCES  (NONE)                                                          
C***ROUTINES CALLED  (NONE)                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   790801  DATE WRITTEN                                                        
C   861211  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C***END PROLOGUE  FDUMP                                                         
C***FIRST EXECUTABLE STATEMENT  FDUMP                                           
      RETURN                                                                    
      END                                                                       
      FUNCTION J4SAVE (IWHICH, IVALUE, ISET)                                    
C***BEGIN PROLOGUE  J4SAVE                                                      
C***SUBSIDIARY                                                                  
C***PURPOSE  Save or recall global variables needed by error                    
C            handling routines.                                                 
C***LIBRARY   SLATEC (XERROR)                                                   
C***TYPE      INTEGER (J4SAVE-I)                                                
C***KEYWORDS  ERROR MESSAGES, ERROR NUMBER, RECALL, SAVE, XERROR                
C***AUTHOR  Jones, R. E., (SNLA)                                                
C***DESCRIPTION                                                                 
C                                                                               
C     Abstract                                                                  
C        J4SAVE saves and recalls several global variables needed               
C        by the library error handling routines.                                
C                                                                               
C     Description of Parameters                                                 
C      --Input--                                                                
C        IWHICH - Index of item desired.                                        
C                = 1 Refers to current error number.                            
C                = 2 Refers to current error control flag.                      
C                = 3 Refers to current unit number to which error               
C                    messages are to be sent.  (0 means use standard.)          
C                = 4 Refers to the maximum number of times any                  
C                     message is to be printed (as set by XERMAX).              
C                = 5 Refers to the total number of units to which               
C                     each error message is to be written.                      
C                = 6 Refers to the 2nd unit for error messages                  
C                = 7 Refers to the 3rd unit for error messages                  
C                = 8 Refers to the 4th unit for error messages                  
C                = 9 Refers to the 5th unit for error messages                  
C        IVALUE - The value to be set for the IWHICH-th parameter,              
C                 if ISET is .TRUE. .                                           
C        ISET   - If ISET=.TRUE., the IWHICH-th parameter will BE               
C                 given the value, IVALUE.  If ISET=.FALSE., the                
C                 IWHICH-th parameter will be unchanged, and IVALUE             
C                 is a dummy parameter.                                         
C      --Output--                                                               
C        The (old) value of the IWHICH-th parameter will be returned            
C        in the function value, J4SAVE.                                         
C                                                                               
C***SEE ALSO  XERMSG                                                            
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  (NONE)                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   790801  DATE WRITTEN                                                        
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900205  Minor modifications to prologue.  (WRB)                             
C   900402  Added TYPE section.  (WRB)                                          
C   910411  Added KEYWORDS section.  (WRB)                                      
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  J4SAVE                                                        
      LOGICAL ISET                                                              
      INTEGER IPARAM(9)                                                         
      SAVE IPARAM                                                               
      DATA IPARAM(1),IPARAM(2),IPARAM(3),IPARAM(4)/0,2,0,10/                    
      DATA IPARAM(5)/1/                                                         
      DATA IPARAM(6),IPARAM(7),IPARAM(8),IPARAM(9)/0,0,0,0/                     
C***FIRST EXECUTABLE STATEMENT  J4SAVE                                          
      J4SAVE = IPARAM(IWHICH)                                                   
      IF (ISET) IPARAM(IWHICH) = IVALUE                                         
      RETURN                                                                    
      END                                                                       
      SUBROUTINE XERCNT (LIBRAR, SUBROU, MESSG, NERR, LEVEL, KONTRL)            
C***BEGIN PROLOGUE  XERCNT                                                      
C***SUBSIDIARY                                                                  
C***PURPOSE  Allow user control over handling of errors.                        
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3C                                                               
C***TYPE      ALL (XERCNT-A)                                                    
C***KEYWORDS  ERROR, XERROR                                                     
C***AUTHOR  Jones, R. E., (SNLA)                                                
C***DESCRIPTION                                                                 
C                                                                               
C     Abstract                                                                  
C        Allows user control over handling of individual errors.                
C        Just after each message is recorded, but before it is                  
C        processed any further (i.e., before it is printed or                   
C        a decision to abort is made), a call is made to XERCNT.                
C        If the user has provided his own version of XERCNT, he                 
C        can then override the value of KONTROL used in processing              
C        this message by redefining its value.                                  
C        KONTRL may be set to any value from -2 to 2.                           
C        The meanings for KONTRL are the same as in XSETF, except               
C        that the value of KONTRL changes only for this message.                
C        If KONTRL is set to a value outside the range from -2 to 2,            
C        it will be moved back into that range.                                 
C                                                                               
C     Description of Parameters                                                 
C                                                                               
C      --Input--                                                                
C        LIBRAR - the library that the routine is in.                           
C        SUBROU - the subroutine that XERMSG is being called from               
C        MESSG  - the first 20 characters of the error message.                 
C        NERR   - same as in the call to XERMSG.                                
C        LEVEL  - same as in the call to XERMSG.                                
C        KONTRL - the current value of the control flag as set                  
C                 by a call to XSETF.                                           
C                                                                               
C      --Output--                                                               
C        KONTRL - the new value of KONTRL.  If KONTRL is not                    
C                 defined, it will remain at its original value.                
C                 This changed value of control affects only                    
C                 the current occurrence of the current message.                
C                                                                               
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  (NONE)                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   790801  DATE WRITTEN                                                        
C   861211  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900206  Routine changed from user-callable to subsidiary.  (WRB)            
C   900510  Changed calling sequence to include LIBRARY and SUBROUTINE          
C           names, changed routine name from XERCTL to XERCNT.  (RWC)           
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  XERCNT                                                        
      CHARACTER*(*) LIBRAR, SUBROU, MESSG                                       
C***FIRST EXECUTABLE STATEMENT  XERCNT                                          
      RETURN                                                                    
      END                                                                       
      SUBROUTINE XERHLT (MESSG)                                                 
C***BEGIN PROLOGUE  XERHLT                                                      
C***SUBSIDIARY                                                                  
C***PURPOSE  Abort program execution and print error message.                   
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3C                                                               
C***TYPE      ALL (XERHLT-A)                                                    
C***KEYWORDS  ABORT PROGRAM EXECUTION, ERROR, XERROR                            
C***AUTHOR  Jones, R. E., (SNLA)                                                
C***DESCRIPTION                                                                 
C                                                                               
C     Abstract                                                                  
C        ***Note*** machine dependent routine                                   
C        XERHLT aborts the execution of the program.                            
C        The error message causing the abort is given in the calling            
C        sequence, in case one needs it for printing on a dayfile,              
C        for example.                                                           
C                                                                               
C     Description of Parameters                                                 
C        MESSG is as in XERMSG.                                                 
C                                                                               
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  (NONE)                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   790801  DATE WRITTEN                                                        
C   861211  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900206  Routine changed from user-callable to subsidiary.  (WRB)            
C   900510  Changed calling sequence to delete length of character              
C           and changed routine name from XERABT to XERHLT.  (RWC)              
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  XERHLT                                                        
      CHARACTER*(*) MESSG                                                       
C***FIRST EXECUTABLE STATEMENT  XERHLT                                          
      STOP                                                                      
      END                                                                       
      SUBROUTINE XERPRN (PREFIX, NPREF, MESSG, NWRAP)                           
C***BEGIN PROLOGUE  XERPRN                                                      
C***SUBSIDIARY                                                                  
C***PURPOSE  Print error messages processed by XERMSG.                          
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3C                                                               
C***TYPE      ALL (XERPRN-A)                                                    
C***KEYWORDS  ERROR MESSAGES, PRINTING, XERROR                                  
C***AUTHOR  Fong, Kirby, (NMFECC at LLNL)                                       
C***DESCRIPTION                                                                 
C                                                                               
C This routine sends one or more lines to each of the (up to five)              
C logical units to which error messages are to be sent.  This routine           
C is called several times by XERMSG, sometimes with a single line to            
C print and sometimes with a (potentially very long) message that may           
C wrap around into multiple lines.                                              
C                                                                               
C PREFIX  Input argument of type CHARACTER.  This argument contains             
C         characters to be put at the beginning of each line before             
C         the body of the message.  No more than 16 characters of               
C         PREFIX will be used.                                                  
C                                                                               
C NPREF   Input argument of type INTEGER.  This argument is the number          
C         of characters to use from PREFIX.  If it is negative, the             
C         intrinsic function LEN is used to determine its length.  If           
C         it is zero, PREFIX is not used.  If it exceeds 16 or if               
C         LEN(PREFIX) exceeds 16, only the first 16 characters will be          
C         used.  If NPREF is positive and the length of PREFIX is less          
C         than NPREF, a copy of PREFIX extended with blanks to length           
C         NPREF will be used.                                                   
C                                                                               
C MESSG   Input argument of type CHARACTER.  This is the text of a              
C         message to be printed.  If it is a long message, it will be           
C         broken into pieces for printing on multiple lines.  Each line         
C         will start with the appropriate prefix and be followed by a           
C         piece of the message.  NWRAP is the number of characters per          
C         piece; that is, after each NWRAP characters, we break and             
C         start a new line.  In addition the characters '$$' embedded           
C         in MESSG are a sentinel for a new line.  The counting of              
C         characters up to NWRAP starts over for each new line.  The            
C         value of NWRAP typically used by XERMSG is 72 since many              
C         older error messages in the SLATEC Library are laid out to            
C         rely on wrap-around every 72 characters.                              
C                                                                               
C NWRAP   Input argument of type INTEGER.  This gives the maximum size          
C         piece into which to break MESSG for printing on multiple              
C         lines.  An embedded '$$' ends a line, and the count restarts          
C         at the following character.  If a line break does not occur           
C         on a blank (it would split a word) that word is moved to the          
C         next line.  Values of NWRAP less than 16 will be treated as           
C         16.  Values of NWRAP greater than 132 will be treated as 132.         
C         The actual line length will be NPREF + NWRAP after NPREF has          
C         been adjusted to fall between 0 and 16 and NWRAP has been             
C         adjusted to fall between 16 and 132.                                  
C                                                                               
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  I1MACH, XGETUA                                             
C***REVISION HISTORY  (YYMMDD)                                                  
C   880621  DATE WRITTEN                                                        
C   880708  REVISED AFTER THE SLATEC CML SUBCOMMITTEE MEETING OF                
C           JUNE 29 AND 30 TO CHANGE THE NAME TO XERPRN AND TO REWORK           
C           THE HANDLING OF THE NEW LINE SENTINEL TO BEHAVE LIKE THE            
C           SLASH CHARACTER IN FORMAT STATEMENTS.                               
C   890706  REVISED WITH THE HELP OF FRED FRITSCH AND REG CLEMENS TO            
C           STREAMLINE THE CODING AND FIX A BUG THAT CAUSED EXTRA BLANK         
C           LINES TO BE PRINTED.                                                
C   890721  REVISED TO ADD A NEW FEATURE.  A NEGATIVE VALUE OF NPREF            
C           CAUSES LEN(PREFIX) TO BE USED AS THE LENGTH.                        
C   891013  REVISED TO CORRECT ERROR IN CALCULATING PREFIX LENGTH.              
C   891214  Prologue converted to Version 4.0 format.  (WRB)                    
C   900510  Added code to break messages between words.  (RWC)                  
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  XERPRN                                                        
      CHARACTER*(*) PREFIX, MESSG                                               
      INTEGER NPREF, NWRAP                                                      
      CHARACTER*148 CBUFF                                                       
      INTEGER IU(5), NUNIT                                                      
      CHARACTER*2 NEWLIN                                                        
      PARAMETER (NEWLIN = '$$')                                                 
C***FIRST EXECUTABLE STATEMENT  XERPRN                                          
      CALL XGETUA(IU,NUNIT)                                                     
C                                                                               
C       A ZERO VALUE FOR A LOGICAL UNIT NUMBER MEANS TO USE THE STANDARD        
C       ERROR MESSAGE UNIT INSTEAD.  I1MACH(4) RETRIEVES THE STANDARD           
C       ERROR MESSAGE UNIT.                                                     
C                                                                               
      N = I1MACH(4)                                                             
      DO 10 I=1,NUNIT                                                           
         IF (IU(I) .EQ. 0) IU(I) = N                                            
   10 CONTINUE                                                                  
C                                                                               
C       LPREF IS THE LENGTH OF THE PREFIX.  THE PREFIX IS PLACED AT THE         
C       BEGINNING OF CBUFF, THE CHARACTER BUFFER, AND KEPT THERE DURING         
C       THE REST OF THIS ROUTINE.                                               
C                                                                               
      IF ( NPREF .LT. 0 ) THEN                                                  
         LPREF = LEN(PREFIX)                                                    
      ELSE                                                                      
         LPREF = NPREF                                                          
      ENDIF                                                                     
      LPREF = MIN(16, LPREF)                                                    
      IF (LPREF .NE. 0) CBUFF(1:LPREF) = PREFIX                                 
C                                                                               
C       LWRAP IS THE MAXIMUM NUMBER OF CHARACTERS WE WANT TO TAKE AT ONE        
C       TIME FROM MESSG TO PRINT ON ONE LINE.                                   
C                                                                               
      LWRAP = MAX(16, MIN(132, NWRAP))                                          
C                                                                               
C       SET LENMSG TO THE LENGTH OF MESSG, IGNORE ANY TRAILING BLANKS.          
C                                                                               
      LENMSG = LEN(MESSG)                                                       
      N = LENMSG                                                                
      DO 20 I=1,N                                                               
         IF (MESSG(LENMSG:LENMSG) .NE. ' ') GO TO 30                            
         LENMSG = LENMSG - 1                                                    
   20 CONTINUE                                                                  
   30 CONTINUE                                                                  
C                                                                               
C       IF THE MESSAGE IS ALL BLANKS, THEN PRINT ONE BLANK LINE.                
C                                                                               
      IF (LENMSG .EQ. 0) THEN                                                   
         CBUFF(LPREF+1:LPREF+1) = ' '                                           
         DO 40 I=1,NUNIT                                                        
            WRITE(IU(I), '(A)') CBUFF(1:LPREF+1)                                
   40    CONTINUE                                                               
         RETURN                                                                 
      ENDIF                                                                     
C                                                                               
C       SET NEXTC TO THE POSITION IN MESSG WHERE THE NEXT SUBSTRING             
C       STARTS.  FROM THIS POSITION WE SCAN FOR THE NEW LINE SENTINEL.          
C       WHEN NEXTC EXCEEDS LENMSG, THERE IS NO MORE TO PRINT.                   
C       WE LOOP BACK TO LABEL 50 UNTIL ALL PIECES HAVE BEEN PRINTED.            
C                                                                               
C       WE LOOK FOR THE NEXT OCCURRENCE OF THE NEW LINE SENTINEL.  THE          
C       INDEX INTRINSIC FUNCTION RETURNS ZERO IF THERE IS NO OCCURRENCE         
C       OR IF THE LENGTH OF THE FIRST ARGUMENT IS LESS THAN THE LENGTH          
C       OF THE SECOND ARGUMENT.                                                 
C                                                                               
C       THERE ARE SEVERAL CASES WHICH SHOULD BE CHECKED FOR IN THE              
C       FOLLOWING ORDER.  WE ARE ATTEMPTING TO SET LPIECE TO THE NUMBER         
C       OF CHARACTERS THAT SHOULD BE TAKEN FROM MESSG STARTING AT               
C       POSITION NEXTC.                                                         
C                                                                               
C       LPIECE .EQ. 0   THE NEW LINE SENTINEL DOES NOT OCCUR IN THE             
C                       REMAINDER OF THE CHARACTER STRING.  LPIECE              
C                       SHOULD BE SET TO LWRAP OR LENMSG+1-NEXTC,               
C                       WHICHEVER IS LESS.                                      
C                                                                               
C       LPIECE .EQ. 1   THE NEW LINE SENTINEL STARTS AT MESSG(NEXTC:            
C                       NEXTC).  LPIECE IS EFFECTIVELY ZERO, AND WE             
C                       PRINT NOTHING TO AVOID PRODUCING UNNECESSARY            
C                       BLANK LINES.  THIS TAKES CARE OF THE SITUATION          
C                       WHERE THE LIBRARY ROUTINE HAS A MESSAGE OF              
C                       EXACTLY 72 CHARACTERS FOLLOWED BY A NEW LINE            
C                       SENTINEL FOLLOWED BY MORE CHARACTERS.  NEXTC            
C                       SHOULD BE INCREMENTED BY 2.                             
C                                                                               
C       LPIECE .GT. LWRAP+1  REDUCE LPIECE TO LWRAP.                            
C                                                                               
C       ELSE            THIS LAST CASE MEANS 2 .LE. LPIECE .LE. LWRAP+1         
C                       RESET LPIECE = LPIECE-1.  NOTE THAT THIS                
C                       PROPERLY HANDLES THE END CASE WHERE LPIECE .EQ.         
C                       LWRAP+1.  THAT IS, THE SENTINEL FALLS EXACTLY           
C                       AT THE END OF A LINE.                                   
C                                                                               
      NEXTC = 1                                                                 
   50 LPIECE = INDEX(MESSG(NEXTC:LENMSG), NEWLIN)                               
      IF (LPIECE .EQ. 0) THEN                                                   
C                                                                               
C       THERE WAS NO NEW LINE SENTINEL FOUND.                                   
C                                                                               
         IDELTA = 0                                                             
         LPIECE = MIN(LWRAP, LENMSG+1-NEXTC)                                    
         IF (LPIECE .LT. LENMSG+1-NEXTC) THEN                                   
            DO 52 I=LPIECE+1,2,-1                                               
               IF (MESSG(NEXTC+I-1:NEXTC+I-1) .EQ. ' ') THEN                    
                  LPIECE = I-1                                                  
                  IDELTA = 1                                                    
                  GOTO 54                                                       
               ENDIF                                                            
   52       CONTINUE                                                            
         ENDIF                                                                  
   54    CBUFF(LPREF+1:LPREF+LPIECE) = MESSG(NEXTC:NEXTC+LPIECE-1)              
         NEXTC = NEXTC + LPIECE + IDELTA                                        
      ELSEIF (LPIECE .EQ. 1) THEN                                               
C                                                                               
C       WE HAVE A NEW LINE SENTINEL AT MESSG(NEXTC:NEXTC+1).                    
C       DON'T PRINT A BLANK LINE.                                               
C                                                                               
         NEXTC = NEXTC + 2                                                      
         GO TO 50                                                               
      ELSEIF (LPIECE .GT. LWRAP+1) THEN                                         
C                                                                               
C       LPIECE SHOULD BE SET DOWN TO LWRAP.                                     
C                                                                               
         IDELTA = 0                                                             
         LPIECE = LWRAP                                                         
         DO 56 I=LPIECE+1,2,-1                                                  
            IF (MESSG(NEXTC+I-1:NEXTC+I-1) .EQ. ' ') THEN                       
               LPIECE = I-1                                                     
               IDELTA = 1                                                       
               GOTO 58                                                          
            ENDIF                                                               
   56    CONTINUE                                                               
   58    CBUFF(LPREF+1:LPREF+LPIECE) = MESSG(NEXTC:NEXTC+LPIECE-1)              
         NEXTC = NEXTC + LPIECE + IDELTA                                        
      ELSE                                                                      
C                                                                               
C       IF WE ARRIVE HERE, IT MEANS 2 .LE. LPIECE .LE. LWRAP+1.                 
C       WE SHOULD DECREMENT LPIECE BY ONE.                                      
C                                                                               
         LPIECE = LPIECE - 1                                                    
         CBUFF(LPREF+1:LPREF+LPIECE) = MESSG(NEXTC:NEXTC+LPIECE-1)              
         NEXTC  = NEXTC + LPIECE + 2                                            
      ENDIF                                                                     
C                                                                               
C       PRINT                                                                   
C                                                                               
      DO 60 I=1,NUNIT                                                           
         WRITE(IU(I), '(A)') CBUFF(1:LPREF+LPIECE)                              
   60 CONTINUE                                                                  
C                                                                               
      IF (NEXTC .LE. LENMSG) GO TO 50                                           
      RETURN                                                                    
      END                                                                       
      SUBROUTINE XERSVE (LIBRAR, SUBROU, MESSG, KFLAG, NERR, LEVEL,             
     +   ICOUNT)                                                                
C***BEGIN PROLOGUE  XERSVE                                                      
C***SUBSIDIARY                                                                  
C***PURPOSE  Record that an error has occurred.                                 
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3                                                                
C***TYPE      ALL (XERSVE-A)                                                    
C***KEYWORDS  ERROR, XERROR                                                     
C***AUTHOR  Jones, R. E., (SNLA)                                                
C***DESCRIPTION                                                                 
C                                                                               
C *Usage:                                                                       
C                                                                               
C        INTEGER  KFLAG, NERR, LEVEL, ICOUNT                                    
C        CHARACTER * (len) LIBRAR, SUBROU, MESSG                                
C                                                                               
C        CALL XERSVE (LIBRAR, SUBROU, MESSG, KFLAG, NERR, LEVEL, ICOUNT)        
C                                                                               
C *Arguments:                                                                   
C                                                                               
C        LIBRAR :IN    is the library that the message is from.                 
C        SUBROU :IN    is the subroutine that the message is from.              
C        MESSG  :IN    is the message to be saved.                              
C        KFLAG  :IN    indicates the action to be performed.                    
C                      when KFLAG > 0, the message in MESSG is saved.           
C                      when KFLAG=0 the tables will be dumped and               
C                      cleared.                                                 
C                      when KFLAG < 0, the tables will be dumped and            
C                      not cleared.                                             
C        NERR   :IN    is the error number.                                     
C        LEVEL  :IN    is the error severity.                                   
C        ICOUNT :OUT   the number of times this message has been seen,          
C                      or zero if the table has overflowed and does not         
C                      contain this message specifically.  When KFLAG=0,        
C                      ICOUNT will not be altered.                              
C                                                                               
C *Description:                                                                 
C                                                                               
C   Record that this error occurred and possibly dump and clear the             
C   tables.                                                                     
C                                                                               
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  I1MACH, XGETUA                                             
C***REVISION HISTORY  (YYMMDD)                                                  
C   800319  DATE WRITTEN                                                        
C   861211  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900413  Routine modified to remove reference to KFLAG.  (WRB)               
C   900510  Changed to add LIBRARY NAME and SUBROUTINE to calling               
C           sequence, use IF-THEN-ELSE, make number of saved entries            
C           easily changeable, changed routine name from XERSAV to              
C           XERSVE.  (RWC)                                                      
C   910626  Added LIBTAB and SUBTAB to SAVE statement.  (BKS)                   
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  XERSVE                                                        
      PARAMETER (LENTAB=10)                                                     
      INTEGER LUN(5)                                                            
      CHARACTER*(*) LIBRAR, SUBROU, MESSG                                       
      CHARACTER*8  LIBTAB(LENTAB), SUBTAB(LENTAB), LIB, SUB                     
      CHARACTER*20 MESTAB(LENTAB), MES                                          
      DIMENSION NERTAB(LENTAB), LEVTAB(LENTAB), KOUNT(LENTAB)                   
      SAVE LIBTAB, SUBTAB, MESTAB, NERTAB, LEVTAB, KOUNT, KOUNTX, NMSG          
      DATA KOUNTX/0/, NMSG/0/                                                   
C***FIRST EXECUTABLE STATEMENT  XERSVE                                          
C                                                                               
      IF (KFLAG.LE.0) THEN                                                      
C                                                                               
C        Dump the table.                                                        
C                                                                               
         IF (NMSG.EQ.0) RETURN                                                  
C                                                                               
C        Print to each unit.                                                    
C                                                                               
         CALL XGETUA (LUN, NUNIT)                                               
         DO 20 KUNIT = 1,NUNIT                                                  
            IUNIT = LUN(KUNIT)                                                  
            IF (IUNIT.EQ.0) IUNIT = I1MACH(4)                                   
C                                                                               
C           Print the table header.                                             
C                                                                               
            WRITE (IUNIT,9000)                                                  
C                                                                               
C           Print body of table.                                                
C                                                                               
            DO 10 I = 1,NMSG                                                    
               WRITE (IUNIT,9010) LIBTAB(I), SUBTAB(I), MESTAB(I),              
     *            NERTAB(I),LEVTAB(I),KOUNT(I)                                  
   10       CONTINUE                                                            
C                                                                               
C           Print number of other errors.                                       
C                                                                               
            IF (KOUNTX.NE.0) WRITE (IUNIT,9020) KOUNTX                          
            WRITE (IUNIT,9030)                                                  
   20    CONTINUE                                                               
C                                                                               
C        Clear the error tables.                                                
C                                                                               
         IF (KFLAG.EQ.0) THEN                                                   
            NMSG = 0                                                            
            KOUNTX = 0                                                          
         ENDIF                                                                  
      ELSE                                                                      
C                                                                               
C        PROCESS A MESSAGE...                                                   
C        SEARCH FOR THIS MESSG, OR ELSE AN EMPTY SLOT FOR THIS MESSG,           
C        OR ELSE DETERMINE THAT THE ERROR TABLE IS FULL.                        
C                                                                               
         LIB = LIBRAR                                                           
         SUB = SUBROU                                                           
         MES = MESSG                                                            
         DO 30 I = 1,NMSG                                                       
            IF (LIB.EQ.LIBTAB(I) .AND. SUB.EQ.SUBTAB(I) .AND.                   
     *         MES.EQ.MESTAB(I) .AND. NERR.EQ.NERTAB(I) .AND.                   
     *         LEVEL.EQ.LEVTAB(I)) THEN                                         
                  KOUNT(I) = KOUNT(I) + 1                                       
                  ICOUNT = KOUNT(I)                                             
                  RETURN                                                        
            ENDIF                                                               
   30    CONTINUE                                                               
C                                                                               
         IF (NMSG.LT.LENTAB) THEN                                               
C                                                                               
C           Empty slot found for new message.                                   
C                                                                               
            NMSG = NMSG + 1                                                     
            LIBTAB(I) = LIB                                                     
            SUBTAB(I) = SUB                                                     
            MESTAB(I) = MES                                                     
            NERTAB(I) = NERR                                                    
            LEVTAB(I) = LEVEL                                                   
            KOUNT (I) = 1                                                       
            ICOUNT    = 1                                                       
         ELSE                                                                   
C                                                                               
C           Table is full.                                                      
C                                                                               
            KOUNTX = KOUNTX+1                                                   
            ICOUNT = 0                                                          
         ENDIF                                                                  
      ENDIF                                                                     
      RETURN                                                                    
C                                                                               
C     Formats.                                                                  
C                                                                               
 9000 FORMAT ('0          ERROR MESSAGE SUMMARY' /                              
     +   ' LIBRARY    SUBROUTINE MESSAGE START             NERR',               
     +   '     LEVEL     COUNT')                                                
 9010 FORMAT (1X,A,3X,A,3X,A,3I10)                                              
 9020 FORMAT ('0OTHER ERRORS NOT INDIVIDUALLY TABULATED = ', I10)               
 9030 FORMAT (1X)                                                               
      END                                                                       
      SUBROUTINE XGETUA (IUNITA, N)                                             
C***BEGIN PROLOGUE  XGETUA                                                      
C***PURPOSE  Return unit number(s) to which error messages are being            
C            sent.                                                              
C***LIBRARY   SLATEC (XERROR)                                                   
C***CATEGORY  R3C                                                               
C***TYPE      ALL (XGETUA-A)                                                    
C***KEYWORDS  ERROR, XERROR                                                     
C***AUTHOR  Jones, R. E., (SNLA)                                                
C***DESCRIPTION                                                                 
C                                                                               
C     Abstract                                                                  
C        XGETUA may be called to determine the unit number or numbers           
C        to which error messages are being sent.                                
C        These unit numbers may have been set by a call to XSETUN,              
C        or a call to XSETUA, or may be a default value.                        
C                                                                               
C     Description of Parameters                                                 
C      --Output--                                                               
C        IUNIT - an array of one to five unit numbers, depending                
C                on the value of N.  A value of zero refers to the              
C                default unit, as defined by the I1MACH machine                 
C                constant routine.  Only IUNIT(1),...,IUNIT(N) are              
C                defined by XGETUA.  The values of IUNIT(N+1),...,              
C                IUNIT(5) are not defined (for N .LT. 5) or altered             
C                in any way by XGETUA.                                          
C        N     - the number of units to which copies of the                     
C                error messages are being sent.  N will be in the               
C                range from 1 to 5.                                             
C                                                                               
C***REFERENCES  R. E. Jones and D. K. Kahaner, XERROR, the SLATEC               
C                 Error-handling Package, SAND82-0800, Sandia                   
C                 Laboratories, 1982.                                           
C***ROUTINES CALLED  J4SAVE                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   790801  DATE WRITTEN                                                        
C   861211  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  XGETUA                                                        
      DIMENSION IUNITA(5)                                                       
C***FIRST EXECUTABLE STATEMENT  XGETUA                                          
      N = J4SAVE(5,0,.FALSE.)                                                   
      DO 30 I=1,N                                                               
         INDEX = I+4                                                            
         IF (I.EQ.1) INDEX = 3                                                  
         IUNITA(I) = J4SAVE(INDEX,0,.FALSE.)                                    
   30 CONTINUE                                                                  
      RETURN                                                                    
      END                                                                       
      DOUBLE PRECISION FUNCTION DBVALU (T, A, N, K, IDERIV, X, INBV,            
     +   WORK)                                                                  
C***BEGIN PROLOGUE  DBVALU                                                      
C***PURPOSE  Evaluate the B-representation of a B-spline at X for the           
C            function value or any of its derivatives.                          
C***LIBRARY   SLATEC                                                            
C***CATEGORY  E3, K6                                                            
C***TYPE      DOUBLE PRECISION (BVALU-S, DBVALU-D)                              
C***KEYWORDS  DIFFERENTIATION OF B-SPLINE, EVALUATION OF B-SPLINE               
C***AUTHOR  Amos, D. E., (SNLA)                                                 
C***DESCRIPTION                                                                 
C                                                                               
C     Written by Carl de Boor and modified by D. E. Amos                        
C                                                                               
C     Abstract   **** a double precision routine ****                           
C         DBVALU is the BVALUE function of the reference.                       
C                                                                               
C         DBVALU evaluates the B-representation (T,A,N,K) of a B-spline         
C         at X for the function value on IDERIV=0 or any of its                 
C         derivatives on IDERIV=1,2,...,K-1.  Right limiting values             
C         (right derivatives) are returned except at the right end              
C         point X=T(N+1) where left limiting values are computed.  The          
C         spline is defined on T(K) .LE. X .LE. T(N+1).  DBVALU returns         
C         a fatal error message when X is outside of this interval.             
C                                                                               
C         To compute left derivatives or left limiting values at a              
C         knot T(I), replace N by I-1 and set X=T(I), I=K+1,N+1.                
C                                                                               
C         DBVALU calls DINTRV                                                   
C                                                                               
C     Description of Arguments                                                  
C                                                                               
C         Input      T,A,X are double precision                                 
C          T       - knot vector of length N+K                                  
C          A       - B-spline coefficient vector of length N                    
C          N       - number of B-spline coefficients                            
C                    N = sum of knot multiplicities-K                           
C          K       - order of the B-spline, K .GE. 1                            
C          IDERIV  - order of the derivative, 0 .LE. IDERIV .LE. K-1            
C                    IDERIV = 0 returns the B-spline value                      
C          X       - argument, T(K) .LE. X .LE. T(N+1)                          
C          INBV    - an initialization parameter which must be set              
C                    to 1 the first time DBVALU is called.                      
C                                                                               
C         Output     WORK,DBVALU are double precision                           
C          INBV    - INBV contains information for efficient process-           
C                    ing after the initial call and INBV must not               
C                    be changed by the user.  Distinct splines require          
C                    distinct INBV parameters.                                  
C          WORK    - work vector of length 3*K.                                 
C          DBVALU  - value of the IDERIV-th derivative at X                     
C                                                                               
C     Error Conditions                                                          
C         An improper input is a fatal error                                    
C                                                                               
C***REFERENCES  Carl de Boor, Package for calculating with B-splines,           
C                 SIAM Journal on Numerical Analysis 14, 3 (June 1977),         
C                 pp. 441-472.                                                  
C***ROUTINES CALLED  DINTRV, XERMSG                                             
C***REVISION HISTORY  (YYMMDD)                                                  
C   800901  DATE WRITTEN                                                        
C   890831  Modified array declarations.  (WRB)                                 
C   890911  Removed unnecessary intrinsics.  (WRB)                              
C   890911  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   900315  CALLs to XERROR changed to CALLs to XERMSG.  (THJ)                  
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  DBVALU                                                        
C                                                                               
      INTEGER I,IDERIV,IDERP1,IHI,IHMKMJ,ILO,IMK,IMKPJ, INBV, IPJ,              
     1 IP1, IP1MJ, J, JJ, J1, J2, K, KMIDER, KMJ, KM1, KPK, MFLAG, N            
      DOUBLE PRECISION A, FKMJ, T, WORK, X                                      
      DIMENSION T(*), A(*), WORK(*)                                             
C***FIRST EXECUTABLE STATEMENT  DBVALU                                          
      DBVALU = 0.0D0                                                            
      IF(K.LT.1) GO TO 102                                                      
      IF(N.LT.K) GO TO 101                                                      
      IF(IDERIV.LT.0 .OR. IDERIV.GE.K) GO TO 110                                
      KMIDER = K - IDERIV                                                       
C                                                                               
C *** FIND *I* IN (K,N) SUCH THAT T(I) .LE. X .LT. T(I+1)                       
C     (OR, .LE. T(I+1) IF T(I) .LT. T(I+1) = T(N+1)).                           
      KM1 = K - 1                                                               
      CALL DINTRV(T, N+1, X, INBV, I, MFLAG)                                    
      IF (X.LT.T(K)) GO TO 120                                                  
      IF (MFLAG.EQ.0) GO TO 20                                                  
      IF (X.GT.T(I)) GO TO 130                                                  
   10 IF (I.EQ.K) GO TO 140                                                     
      I = I - 1                                                                 
      IF (X.EQ.T(I)) GO TO 10                                                   
C                                                                               
C *** DIFFERENCE THE COEFFICIENTS *IDERIV* TIMES                                
C     WORK(I) = AJ(I), WORK(K+I) = DP(I), WORK(K+K+I) = DM(I), I=1.K            
C                                                                               
   20 IMK = I - K                                                               
      DO 30 J=1,K                                                               
        IMKPJ = IMK + J                                                         
        WORK(J) = A(IMKPJ)                                                      
   30 CONTINUE                                                                  
      IF (IDERIV.EQ.0) GO TO 60                                                 
      DO 50 J=1,IDERIV                                                          
        KMJ = K - J                                                             
        FKMJ = KMJ                                                              
        DO 40 JJ=1,KMJ                                                          
          IHI = I + JJ                                                          
          IHMKMJ = IHI - KMJ                                                    
          WORK(JJ) = (WORK(JJ+1)-WORK(JJ))/(T(IHI)-T(IHMKMJ))*FKMJ              
   40   CONTINUE                                                                
   50 CONTINUE                                                                  
C                                                                               
C *** COMPUTE VALUE AT *X* IN (T(I),(T(I+1)) OF IDERIV-TH DERIVATIVE,           
C     GIVEN ITS RELEVANT B-SPLINE COEFF. IN AJ(1),...,AJ(K-IDERIV).             
   60 IF (IDERIV.EQ.KM1) GO TO 100                                              
      IP1 = I + 1                                                               
      KPK = K + K                                                               
      J1 = K + 1                                                                
      J2 = KPK + 1                                                              
      DO 70 J=1,KMIDER                                                          
        IPJ = I + J                                                             
        WORK(J1) = T(IPJ) - X                                                   
        IP1MJ = IP1 - J                                                         
        WORK(J2) = X - T(IP1MJ)                                                 
        J1 = J1 + 1                                                             
        J2 = J2 + 1                                                             
   70 CONTINUE                                                                  
      IDERP1 = IDERIV + 1                                                       
      DO 90 J=IDERP1,KM1                                                        
        KMJ = K - J                                                             
        ILO = KMJ                                                               
        DO 80 JJ=1,KMJ                                                          
          WORK(JJ) = (WORK(JJ+1)*WORK(KPK+ILO)+WORK(JJ)                         
     1              *WORK(K+JJ))/(WORK(KPK+ILO)+WORK(K+JJ))                     
          ILO = ILO - 1                                                         
   80   CONTINUE                                                                
   90 CONTINUE                                                                  
  100 DBVALU = WORK(1)                                                          
      RETURN                                                                    
C                                                                               
C                                                                               
  101 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBVALU', 'N DOES NOT SATISFY N.GE.K', 2,          
     +   1)                                                                     
      RETURN                                                                    
  102 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBVALU', 'K DOES NOT SATISFY K.GE.1', 2,          
     +   1)                                                                     
      RETURN                                                                    
  110 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBVALU',                                          
     +   'IDERIV DOES NOT SATISFY 0.LE.IDERIV.LT.K', 2, 1)                      
      RETURN                                                                    
  120 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBVALU',                                          
     +   'X IS N0T GREATER THAN OR EQUAL TO T(K)', 2, 1)                        
      RETURN                                                                    
  130 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBVALU',                                          
     +   'X IS NOT LESS THAN OR EQUAL TO T(N+1)', 2, 1)                         
      RETURN                                                                    
  140 CONTINUE                                                                  
      CALL XERMSG ('SLATEC', 'DBVALU',                                          
     +   'A LEFT LIMITING VALUE CANNOT BE OBTAINED AT T(K)', 2, 1)              
      RETURN                                                                    
      END                                                                       
      SUBROUTINE DINTRV (XT, LXT, X, ILO, ILEFT, MFLAG)                         
C***BEGIN PROLOGUE  DINTRV                                                      
C***PURPOSE  Compute the largest integer ILEFT in 1 .LE. ILEFT .LE. LXT         
C            such that XT(ILEFT) .LE. X where XT(*) is a subdivision of         
C            the X interval.                                                    
C***LIBRARY   SLATEC                                                            
C***CATEGORY  E3, K6                                                            
C***TYPE      DOUBLE PRECISION (INTRV-S, DINTRV-D)                              
C***KEYWORDS  B-SPLINE, DATA FITTING, INTERPOLATION, SPLINES                    
C***AUTHOR  Amos, D. E., (SNLA)                                                 
C***DESCRIPTION                                                                 
C                                                                               
C     Written by Carl de Boor and modified by D. E. Amos                        
C                                                                               
C     Abstract    **** a double precision routine ****                          
C         DINTRV is the INTERV routine of the reference.                        
C                                                                               
C         DINTRV computes the largest integer ILEFT in 1 .LE. ILEFT .LE.        
C         LXT such that XT(ILEFT) .LE. X where XT(*) is a subdivision of        
C         the X interval.  Precisely,                                           
C                                                                               
C                      X .LT. XT(1)                1         -1                 
C         if  XT(I) .LE. X .LT. XT(I+1)  then  ILEFT=I  , MFLAG=0               
C           XT(LXT) .LE. X                         LXT        1,                
C                                                                               
C         That is, when multiplicities are present in the break point           
C         to the left of X, the largest index is taken for ILEFT.               
C                                                                               
C     Description of Arguments                                                  
C                                                                               
C         Input      XT,X are double precision                                  
C          XT      - XT is a knot or break point vector of length LXT           
C          LXT     - length of the XT vector                                    
C          X       - argument                                                   
C          ILO     - an initialization parameter which must be set              
C                    to 1 the first time the spline array XT is                 
C                    processed by DINTRV.                                       
C                                                                               
C         Output                                                                
C          ILO     - ILO contains information for efficient process-            
C                    ing after the initial call and ILO must not be             
C                    changed by the user.  Distinct splines require             
C                    distinct ILO parameters.                                   
C          ILEFT   - largest integer satisfying XT(ILEFT) .LE. X                
C          MFLAG   - signals when X lies out of bounds                          
C                                                                               
C     Error Conditions                                                          
C         None                                                                  
C                                                                               
C***REFERENCES  Carl de Boor, Package for calculating with B-splines,           
C                 SIAM Journal on Numerical Analysis 14, 3 (June 1977),         
C                 pp. 441-472.                                                  
C***ROUTINES CALLED  (NONE)                                                     
C***REVISION HISTORY  (YYMMDD)                                                  
C   800901  DATE WRITTEN                                                        
C   890831  Modified array declarations.  (WRB)                                 
C   890831  REVISION DATE from Version 3.2                                      
C   891214  Prologue converted to Version 4.0 format.  (BAB)                    
C   920501  Reformatted the REFERENCES section.  (WRB)                          
C***END PROLOGUE  DINTRV                                                        
C                                                                               
      INTEGER IHI, ILEFT, ILO, ISTEP, LXT, MFLAG, MIDDLE                        
      DOUBLE PRECISION X, XT                                                    
      DIMENSION XT(*)                                                           
C***FIRST EXECUTABLE STATEMENT  DINTRV                                          
      IHI = ILO + 1                                                             
      IF (IHI.LT.LXT) GO TO 10                                                  
      IF (X.GE.XT(LXT)) GO TO 110                                               
      IF (LXT.LE.1) GO TO 90                                                    
      ILO = LXT - 1                                                             
      IHI = LXT                                                                 
C                                                                               
   10 IF (X.GE.XT(IHI)) GO TO 40                                                
      IF (X.GE.XT(ILO)) GO TO 100                                               
C                                                                               
C *** NOW X .LT. XT(IHI) . FIND LOWER BOUND                                     
      ISTEP = 1                                                                 
   20 IHI = ILO                                                                 
      ILO = IHI - ISTEP                                                         
      IF (ILO.LE.1) GO TO 30                                                    
      IF (X.GE.XT(ILO)) GO TO 70                                                
      ISTEP = ISTEP*2                                                           
      GO TO 20                                                                  
   30 ILO = 1                                                                   
      IF (X.LT.XT(1)) GO TO 90                                                  
      GO TO 70                                                                  
C *** NOW X .GE. XT(ILO) . FIND UPPER BOUND                                     
   40 ISTEP = 1                                                                 
   50 ILO = IHI                                                                 
      IHI = ILO + ISTEP                                                         
      IF (IHI.GE.LXT) GO TO 60                                                  
      IF (X.LT.XT(IHI)) GO TO 70                                                
      ISTEP = ISTEP*2                                                           
      GO TO 50                                                                  
   60 IF (X.GE.XT(LXT)) GO TO 110                                               
      IHI = LXT                                                                 
C                                                                               
C *** NOW XT(ILO) .LE. X .LT. XT(IHI) . NARROW THE INTERVAL                     
   70 MIDDLE = (ILO+IHI)/2                                                      
      IF (MIDDLE.EQ.ILO) GO TO 100                                              
C     NOTE. IT IS ASSUMED THAT MIDDLE = ILO IN CASE IHI = ILO+1                 
      IF (X.LT.XT(MIDDLE)) GO TO 80                                             
      ILO = MIDDLE                                                              
      GO TO 70                                                                  
   80 IHI = MIDDLE                                                              
      GO TO 70                                                                  
C *** SET OUTPUT AND RETURN                                                     
   90 MFLAG = -1                                                                
      ILEFT = 1                                                                 
      RETURN                                                                    
  100 MFLAG = 0                                                                 
      ILEFT = ILO                                                               
      RETURN                                                                    
  110 MFLAG = 1                                                                 
      ILEFT = LXT                                                               
      RETURN                                                                    
      END                                                                       
